@@ -1,8 +1,9 @@
+import logging
+
 from metadrive.base_class.base_runnable import BaseRunnable
 from metadrive.engine.engine_utils import get_engine, get_global_config
-from metadrive.utils import import_pygame
 
-pygame = import_pygame()
+logger = logging.getLogger(__name__)
 
 
 class BaseMap(BaseRunnable):
@@ -47,13 +48,15 @@ class BaseMap(BaseRunnable):
         self.blocks = []
 
         # Generate map and insert blocks
-        self.engine = get_engine()
+        # self.engine = get_engine()
         self._generate()
         assert self.blocks, "The generate methods does not fill blocks!"
 
         #  a trick to optimize performance
         self.spawn_roads = None
         self.detach_from_world()
+        if self.engine.global_config["show_coordinates"]:
+            self.show_coordinates()
 
     def _generate(self):
         """Key function! Please overwrite it! This func aims at fill the self.road_network adn self.blocks"""
@@ -80,12 +83,17 @@ class BaseMap(BaseRunnable):
 
     def destroy(self):
         self.detach_from_world()
+
         for block in self.blocks:
             block.destroy()
-        self.blocks = None
-        self.road_network.destroy()
+        self.blocks = []
+
+        if self.road_network is not None:
+            self.road_network.destroy()
         self.road_network = None
+
         self.spawn_roads = None
+
         super(BaseMap, self).destroy()
 
     @property
@@ -95,3 +103,10 @@ class BaseMap(BaseRunnable):
     def get_center_point(self):
         x_min, x_max, y_min, y_max = self.road_network.get_bounding_box()
         return (x_max + x_min) / 2, (y_max + y_min) / 2
+
+    def __del__(self):
+        # self.destroy()
+        logger.debug("{} is being deleted.".format(type(self)))
+
+    def show_coordinates(self):
+        pass
