@@ -1,5 +1,7 @@
 from typing import Union
 
+from panda3d.core import NodePath
+
 from metadrive.component.algorithm.BIG import BIG
 from metadrive.component.algorithm.blocks_prob_dist import PGBlockDistConfig
 from metadrive.component.map.base_map import BaseMap
@@ -7,7 +9,6 @@ from metadrive.component.pgblock.first_block import FirstPGBlock
 from metadrive.component.pgblock.pg_block import PGBlock
 from metadrive.component.road_network.node_road_network import NodeRoadNetwork
 from metadrive.engine.core.physics_world import PhysicsWorld
-from panda3d.core import NodePath
 
 
 class NextStep:
@@ -69,6 +70,8 @@ class CityBIG(BIG):
             block_types = PGBlockDistConfig.all_blocks()
             block_probabilities = PGBlockDistConfig.block_probability()
             block_type = self.np_random.choice(block_types, p=block_probabilities)
+            from metadrive.utils.registry import get_metadrive_class
+            block_type = get_metadrive_class(block_type)
         else:
             type_id = self._block_sequence[len(self.blocks)]
             block_type = PGBlockDistConfig.get_block(type_id)
