@@ -1,14 +1,14 @@
-from typing import List
 import copy
+from typing import List
+
+from panda3d.core import NodePath
 
 from metadrive.component.algorithm.BIG import BigGenerateMethod, BIG
-from metadrive.component.algorithm.blocks_prob_dist import PGBlockDistConfig
 from metadrive.component.map.base_map import BaseMap
 from metadrive.component.pgblock.first_block import FirstPGBlock
 from metadrive.component.road_network.node_road_network import NodeRoadNetwork
 from metadrive.engine.core.physics_world import PhysicsWorld
 from metadrive.utils import Config
-from panda3d.core import NodePath
 
 
 def parse_map_config(easy_map_config, new_map_config, default_config):
@@ -72,6 +72,7 @@ class PGMap(BaseMap):
         )
         big_map.generate(self._config[self.GENERATE_TYPE], self._config[self.GENERATE_CONFIG])
         self.blocks = big_map.blocks
+        del big_map
 
     def _config_generate(self, blocks_config: List, parent_node_path: NodePath, physics_world: PhysicsWorld):
         assert len(self.road_network.graph) == 0, "These Map is not empty, please create a new map to read config"
@@ -114,3 +115,10 @@ class PGMap(BaseMap):
 
         saved_data = copy.deepcopy({self.BLOCK_SEQUENCE: map_config, "map_config": self.config.copy()})
         return saved_data
+
+    def show_coordinates(self):
+        lanes = []
+        for to_ in self.road_network.graph.values():
+            for lanes_to_add in to_.values():
+                lanes += lanes_to_add
+        self.engine.show_lane_coordinates(lanes)
