@@ -16,8 +16,8 @@ class TrafficObject(BaseStaticObject):
     """
     Common interface for objects that appear on the road, beside vehicles.
     """
-    NAME = BodyName.Traffic_object
-    COLLISION_GROUP = CollisionGroup.TrafficObject
+    CLASS_NAME = BodyName.Traffic_object
+    COLLISION_MASK = CollisionGroup.TrafficObject
 
     COST_ONCE = True  # cost will give at the first time
 
@@ -29,7 +29,7 @@ class TrafficObject(BaseStaticObject):
         """
         position = lane.position(longitude, lateral)
         heading_theta = lane.heading_theta_at(longitude)
-        assert self.NAME is not None, "Assign a name for this class for finding it easily"
+        assert self.CLASS_NAME is not None, "Assign a name for this class for finding it easily"
         super(TrafficObject, self).__init__(lane, position, heading_theta, random_seed)
         self.crashed = False
 
@@ -43,9 +43,11 @@ class TrafficCone(TrafficObject):
 
     def __init__(self, lane, longitude: float, lateral: float, static: bool = False, random_seed=None):
         super(TrafficCone, self).__init__(lane, longitude, lateral, random_seed)
-        self.add_body(BaseRigidBodyNode(self.name, self.NAME))
+
+        n = BaseRigidBodyNode(self.name, self.CLASS_NAME)
+        self.add_body(n)
+
         self.body.addShape(BulletCylinderShape(self.RADIUS, self.HEIGHT))
-        self.body.setIntoCollideMask(self.COLLISION_GROUP)
         self.set_static(static)
         if self.render:
             model = self.loader.loadModel(AssetLoader.file_path("models", "traffic_cone", "scene.gltf"))
@@ -65,6 +67,14 @@ class TrafficCone(TrafficObject):
     def top_down_color(self):
         return 235, 84, 42
 
+    @property
+    def LENGTH(self):
+        return self.RADIUS
+
+    @property
+    def WIDTH(self):
+        return self.RADIUS
+
 
 class TrafficWarning(TrafficObject):
     """Placed behind the vehicle when it breaks down"""
@@ -75,9 +85,11 @@ class TrafficWarning(TrafficObject):
 
     def __init__(self, lane, longitude: float, lateral: float, static: bool = False, random_seed=None):
         super(TrafficWarning, self).__init__(lane, longitude, lateral, random_seed)
-        self.add_body(BaseRigidBodyNode(self.name, self.NAME))
+
+        n = BaseRigidBodyNode(self.name, self.CLASS_NAME)
+        self.add_body(n)
+
         self.body.addShape(BulletCylinderShape(self.RADIUS, self.HEIGHT))
-        self.body.setIntoCollideMask(self.COLLISION_GROUP)
         self.set_static(static)
         if self.render:
             model = self.loader.loadModel(AssetLoader.file_path("models", "warning", "warning.gltf"))
@@ -94,6 +106,14 @@ class TrafficWarning(TrafficObject):
     def top_down_width(self):
         return self.RADIUS * 2
 
+    @property
+    def LENGTH(self):
+        return self.RADIUS
+
+    @property
+    def WIDTH(self):
+        return self.RADIUS
+
 
 class TrafficBarrier(TrafficObject):
     """A barrier"""
@@ -105,15 +125,28 @@ class TrafficBarrier(TrafficObject):
 
     def __init__(self, lane, longitude: float, lateral: float, static: bool = False, random_seed=None):
         super(TrafficBarrier, self).__init__(lane, longitude, lateral, random_seed)
-        self.add_body(BaseRigidBodyNode(self.name, self.NAME))
-        self.body.addShape(BulletBoxShape((self.WIDTH / 2, self.LENGTH / 2, self.HEIGHT / 2)))
-        self.body.setIntoCollideMask(self.COLLISION_GROUP)
+        n = BaseRigidBodyNode(self.name, self.CLASS_NAME)
+        self.add_body(n)
+
+        self.body.addShape(BulletBoxShape((self.width / 2, self.length / 2, self.height / 2)))
         self.set_static(static)
         if self.render:
             model = self.loader.loadModel(AssetLoader.file_path("models", "barrier", "scene.gltf"))
             model.setH(-90)
             model.reparentTo(self.origin)
 
+    @property
+    def width(self):
+        return self.WIDTH
+
+    @property
+    def length(self):
+        return self.LENGTH
+
+    @property
+    def height(self):
+        return self.HEIGHT
+
     @property
     def top_down_length(self):
         # reverse the direction
