@@ -1,15 +1,19 @@
 import math
+from metadrive.constants import TrafficLightStatus
+from panda3d.core import NodePath
 from collections import deque
 from typing import Union, Optional
-
-import gym
+from metadrive.utils.utils import get_object_from_node
 import numpy as np
 import seaborn as sns
+from panda3d.bullet import BulletVehicle, BulletBoxShape, ZUp
+from panda3d.core import Material, Vec3, TransformState, LQuaternionf
+
 from metadrive.base_class.base_object import BaseObject
 from metadrive.component.lane.abs_lane import AbstractLane
 from metadrive.component.lane.circular_lane import CircularLane
+from metadrive.component.lane.point_lane import PointLane
 from metadrive.component.lane.straight_lane import StraightLane
-from metadrive.component.lane.waypoint_lane import WayPointLane
 from metadrive.component.road_network.node_road_network import NodeRoadNetwork
 from metadrive.component.vehicle_module.depth_camera import DepthCamera
 from metadrive.component.vehicle_module.distance_detector import SideDetector, LaneLineDetector
@@ -30,24 +34,11 @@ from metadrive.utils.math_utils import wrap_to_pi
 from metadrive.utils.scene_utils import ray_localization
 from metadrive.utils.scene_utils import rect_region_detection
 from metadrive.utils.space import VehicleParameterSpace, ParameterSpace
-from panda3d.bullet import BulletVehicle, BulletBoxShape, ZUp
-from panda3d.core import Material, Vec3, TransformState, LQuaternionf
 
 
 class BaseVehicleState:
     def __init__(self):
-        self.crash_vehicle = False
-        self.crash_object = False
-        self.crash_sidewalk = False
-        self.crash_building = False
-
-        # lane line detection
-        self.on_yellow_continuous_line = False
-        self.on_white_continuous_line = False
-        self.on_broken_line = False
-
-        # contact results, a set containing objects type name for rendering
-        self.contact_results = None
+        self.init_state_info()
 
     def init_state_info(self):
         """
@@ -58,11 +49,17 @@ class BaseVehicleState:
         self.crash_sidewalk = False
         self.crash_building = False
 
+        # traffic light
+        self.red_light = False
+        self.yellow_light = False
+        self.green_light = False
+
+        # lane line detection
         self.on_yellow_continuous_line = False
         self.on_white_continuous_line = False
         self.on_broken_line = False
 
-        # contact results
+        # contact results, a set containing objects type name for rendering
         self.contact_results = set()
 
 
@@ -83,24 +80,29 @@ class BaseVehicle(BaseObject, BaseVehicleState):
     MAX_WIDTH = 2.5
     MAX_STEERING = 60
 
-    LENGTH = None
-    WIDTH = None
-    HEIGHT = None
+    # LENGTH = None
+    # WIDTH = None
+    # HEIGHT = None
+
     TIRE_RADIUS = None
+    TIRE_MODEL_CORRECT = 1  # correct model left right error
+    TIRE_TWO_SIDED = False  # tires for some vehicles need two-sided enabled for correctly visualization
     LATERAL_TIRE_TO_CENTER = None
     TIRE_WIDTH = 0.4
     FRONT_WHEELBASE = None
     REAR_WHEELBASE = None
-    MASS = None
+
+    # MASS = None
+
     CHASSIS_TO_WHEEL_AXIS = 0.2
     SUSPENSION_LENGTH = 15
     SUSPENSION_STIFFNESS = 40
 
     # for random color choosing
-    MATERIAL_COLOR_COEFF = 10  # to resist other factors, since other setting may make color dark
-    MATERIAL_METAL_COEFF = 1  # 0-1
+    MATERIAL_COLOR_COEFF = 2  # to resist other factors, since other setting may make color dark
+    MATERIAL_METAL_COEFF = 0.1  # 0-1
     MATERIAL_ROUGHNESS = 0.8  # smaller to make it more smooth, and reflect more light
-    MATERIAL_SHININESS = 1  # 0-128 smaller to make it more smooth, and reflect more light
+    MATERIAL_SHININESS = 128  # 0-128 smaller to make it more smooth, and reflect more light
     MATERIAL_SPECULAR_COLOR = (3, 3, 3, 3)
 
     # control
@@ -116,7 +118,7 @@ class BaseVehicle(BaseObject, BaseVehicleState):
         name: str = None,
         random_seed=None,
         position=None,
-        heading=None
+        heading=None  # In degree!
     ):
         """
         This Vehicle Config is different from self.get_config(), and it is used to define which modules to use, and
@@ -129,7 +131,7 @@ class BaseVehicle(BaseObject, BaseVehicleState):
         assert engine_initialized(), "Please make sure game engine is successfully initialized!"
 
         # NOTE: it is the game engine, not vehicle drivetrain
-        self.engine = get_engine()
+        # self.engine = get_engine()
         BaseObject.__init__(self, name, random_seed, self.engine.global_config["vehicle_config"])
         BaseVehicleState.__init__(self)
         self.update_config(vehicle_config)
@@ -178,7 +180,6 @@ class BaseVehicle(BaseObject, BaseVehicleState):
         self.takeover = False
         self.expert_takeover = False
         self.energy_consumption = 0
-        self.action_space = self.get_action_space_before_init(extra_action_dim=self.config["extra_action_dim"])
         self.break_down = False
 
         # overtake_stat
@@ -189,6 +190,10 @@ class BaseVehicle(BaseObject, BaseVehicleState):
             self.reset(position=position, heading=heading)
 
     def _add_modules_for_vehicle(self, ):
+        """
+        This function is related to the self.update_config, which will create modules if needed for resetting a new
+        vehicle
+        """
         config = self.config
 
         # add routing module
@@ -211,9 +216,58 @@ class BaseVehicle(BaseObject, BaseVehicleState):
         )
 
         # vision modules
-        self.add_image_sensor("rgb_camera", RGBCamera())
-        self.add_image_sensor("mini_map", MiniMap())
-        self.add_image_sensor("depth_camera", DepthCamera())
+        self.setup_sensors()
+
+    def _available_sensors(self):
+        def _main_cam():
+            assert self.engine.main_camera is not None, "Main camera doesn't exist"
+            return self.engine.main_camera
+
+        return {"rgb_camera": RGBCamera, "mini_map": MiniMap, "depth_camera": DepthCamera, "main_camera": _main_cam}
+
+    def setup_sensors(self):
+        if self.engine.global_config["image_observation"]:
+            sensors = self._available_sensors()
+            self.add_image_sensor(self.config["image_source"], sensors[self.config["image_source"]]())
+
+    def get_camera(self, camera_type_str):
+        sensors = self._available_sensors()
+        if camera_type_str not in sensors:
+            raise ValueError("Can not get {}, available type: {}".format(camera_type_str, sensors.keys()))
+        if camera_type_str not in self.image_sensors:
+            self.add_image_sensor(camera_type_str, sensors[camera_type_str]())
+        return self.image_sensors[camera_type_str]
+
+    def _add_modules_for_vehicle_when_reset(self):
+        config = self.config
+
+        # add routing module
+        if self.navigation is None:
+            self.add_navigation()  # default added
+
+        # add distance detector/lidar
+        if self.side_detector is None:
+            self.side_detector = SideDetector(
+                config["side_detector"]["num_lasers"], config["side_detector"]["distance"],
+                self.engine.global_config["vehicle_config"]["show_side_detector"]
+            )
+
+        if self.lane_line_detector is None:
+            self.lane_line_detector = LaneLineDetector(
+                config["lane_line_detector"]["num_lasers"], config["lane_line_detector"]["distance"],
+                self.engine.global_config["vehicle_config"]["show_lane_line_detector"]
+            )
+
+        if self.lidar is None:
+            self.lidar = Lidar(
+                config["lidar"]["num_lasers"], config["lidar"]["distance"],
+                self.engine.global_config["vehicle_config"]["show_lidar"]
+            )
+
+        # vision modules
+        # self.add_image_sensor("rgb_camera", RGBCamera())
+        # self.add_image_sensor("mini_map", MiniMap())
+        # self.add_image_sensor("depth_camera", DepthCamera())
 
     def _init_step_info(self):
         # done info will be initialized every frame
@@ -221,12 +275,9 @@ class BaseVehicle(BaseObject, BaseVehicleState):
         self.out_of_route = False  # re-route is required if is false
         self.on_lane = True  # on lane surface or not
 
-    def _preprocess_action(self, action):
+    @staticmethod
+    def _preprocess_action(action):
         action = safe_clip_for_small_array(action, -1, 1)
-        if self.config["action_check"]:
-            assert self.action_space.contains(action), "Input {} is not compatible with action space {}!".format(
-                action, self.action_space
-            )
         return action, {'raw_action': (action[0], action[1])}
 
     def before_step(self, action=None):
@@ -281,7 +332,7 @@ class BaseVehicle(BaseObject, BaseVehicleState):
         :return: None
         """
         distance = norm(*(self.last_position - self.position)) / 1000  # km
-        step_energy = 3.25 * math.pow(np.e, 0.01 * self.speed) * distance / 100
+        step_energy = 3.25 * math.pow(np.e, 0.01 * self.speed_km_h) * distance / 100
         # step_energy is in Liter, we return mL
         step_energy = step_energy * 1000
         self.energy_consumption += step_energy  # L/100 km
@@ -292,7 +343,7 @@ class BaseVehicle(BaseObject, BaseVehicleState):
         random_seed=None,
         vehicle_config=None,
         position: np.ndarray = None,
-        heading: float = 0.0,
+        heading: float = 0.0,  # In degree!
         *args,
         **kwargs
     ):
@@ -302,10 +353,15 @@ class BaseVehicle(BaseObject, BaseVehicleState):
         else, vehicle will be reset to spawn place
         """
         if random_seed is not None:
+            assert isinstance(random_seed, int)
             self.seed(random_seed)
             self.sample_parameters()
         if vehicle_config is not None:
             self.update_config(vehicle_config)
+
+        # Update some modules that might not be initialized before
+        self._add_modules_for_vehicle_when_reset()
+
         map = self.engine.current_map
 
         if position is not None:
@@ -324,7 +380,7 @@ class BaseVehicle(BaseObject, BaseVehicleState):
         self.spawn_place = position
         heading = -np.deg2rad(heading) - np.pi / 2
         self.set_static(False)
-        self.set_position(position, self.HEIGHT / 2 + 1)
+        self.set_position(position, self.HEIGHT / 2)
         self.origin.setQuat(LQuaternionf(math.cos(heading / 2), 0, 0, math.sin(heading / 2)))
         self.update_map_info(map)
         self.body.clearForces()
@@ -355,6 +411,9 @@ class BaseVehicle(BaseObject, BaseVehicleState):
         if self.config["need_navigation"]:
             assert self.navigation
 
+        if self.config["spawn_velocity"] is not None:
+            self.set_velocity(self.config["spawn_velocity"], in_local_frame=self.config["spawn_velocity_car_frame"])
+
     """------------------------------------------- act -------------------------------------------------"""
 
     def _set_action(self, action):
@@ -380,7 +439,7 @@ class BaseVehicle(BaseObject, BaseVehicleState):
         for wheel_index in range(4):
             if throttle_brake >= 0:
                 self.system.setBrake(2.0, wheel_index)
-                if self.speed > self.max_speed:
+                if self.speed_km_h > self.max_speed_km_h:
                     self.system.applyEngineForce(0.0, wheel_index)
                 else:
                     self.system.applyEngineForce(max_engine_force * throttle_brake, wheel_index)
@@ -399,7 +458,7 @@ class BaseVehicle(BaseObject, BaseVehicleState):
 
     def _dist_to_route_left_right(self):
         # TODO
-        if self.navigation is None:
+        if self.navigation is None or self.navigation.current_ref_lanes is None:
             return 0, 0
         current_reference_lane = self.navigation.current_ref_lanes[0]
         _, lateral_to_reference = current_reference_lane.local_coordinates(self.position)
@@ -415,21 +474,20 @@ class BaseVehicle(BaseObject, BaseVehicleState):
         """
         return (metadrive_heading(self.origin.getH()) - 90) / 180 * math.pi
 
-    @property
-    def speed(self):
-        """
-        km/h
-        """
-        velocity = self.body.get_linear_velocity()
-        speed = norm(velocity[0], velocity[1]) * 3.6
-        return clip(speed, 0.0, 100000.0)
+    # @property
+    # def velocity(self) -> np.ndarray:
+    #     return self.speed * self.velocity_direction
+    #
+    # @property
+    # def velocity_km_h(self) -> np.ndarray:
+    #     return self.speed * self.velocity_direction * 3.6
 
     @property
-    def velocity(self) -> np.ndarray:
-        return self.speed * self.velocity_direction
-
-    @property
-    def velocity_direction(self):
+    def chassis_velocity_direction(self):
+        raise DeprecationWarning(
+            "This API returns the direction of velocity which is approximately heading direction. "
+            "Deprecate it and make things easy"
+        )
         direction = self.system.getForwardVector()
         return np.asarray([direction[0], -direction[1]])
 
@@ -444,7 +502,7 @@ class BaseVehicle(BaseObject, BaseVehicleState):
                 lateral = self.position - target_lane.center
             else:
                 lateral = target_lane.center - self.position
-        elif isinstance(target_lane, WayPointLane):
+        elif isinstance(target_lane, PointLane):
             lateral = target_lane.lateral_direction(target_lane.local_coordinates(self.position)[0])
 
         lateral_norm = norm(lateral[0], lateral[1])
@@ -489,15 +547,29 @@ class BaseVehicle(BaseObject, BaseVehicleState):
 
     """-------------------------------------- for vehicle making ------------------------------------------"""
 
+    @property
+    def LENGTH(self):
+        raise NotImplementedError()
+
+    @property
+    def HEIGHT(self):
+        raise NotImplementedError()
+
+    @property
+    def WIDTH(self):
+        raise NotImplementedError()
+
     def _create_vehicle_chassis(self):
-        self.LENGTH = type(self).LENGTH
-        self.WIDTH = type(self).WIDTH
-        self.HEIGHT = type(self).HEIGHT
-        assert self.LENGTH < BaseVehicle.MAX_LENGTH, "Vehicle is too large!"
-        assert self.WIDTH < BaseVehicle.MAX_WIDTH, "Vehicle is too large!"
+        # self.LENGTH = type(self).LENGTH
+        # self.WIDTH = type(self).WIDTH
+        # self.HEIGHT = type(self).HEIGHT
+
+        # assert self.LENGTH < BaseVehicle.MAX_LENGTH, "Vehicle is too large!"
+        # assert self.WIDTH < BaseVehicle.MAX_WIDTH, "Vehicle is too large!"
 
         chassis = BaseRigidBodyNode(self.name, BodyName.Vehicle)
-        chassis.setIntoCollideMask(CollisionGroup.Vehicle)
+        self._node_path_list.append(chassis)
+
         chassis_shape = BulletBoxShape(Vec3(self.WIDTH / 2, self.LENGTH / 2, self.HEIGHT / 2))
         ts = TransformState.makePos(Vec3(0, 0, self.HEIGHT / 2))
         chassis.addShape(chassis_shape, ts)
@@ -555,12 +627,15 @@ class BaseVehicle(BaseObject, BaseVehicleState):
 
     def _add_wheel(self, pos: Vec3, radius: float, front: bool, left):
         wheel_np = self.origin.attachNewNode("wheel")
+        self._node_path_list.append(wheel_np)
+
         if self.render:
             model = 'right_tire_front.gltf' if front else 'right_tire_back.gltf'
             model_path = AssetLoader.file_path("models", self.path[0], model)
             wheel_model = self.loader.loadModel(model_path)
+            wheel_model.setTwoSided(self.TIRE_TWO_SIDED)
             wheel_model.reparentTo(wheel_np)
-            wheel_model.set_scale(1 if left else -1)
+            wheel_model.set_scale(1 * self.TIRE_MODEL_CORRECT if left else -1 * self.TIRE_MODEL_CORRECT)
         wheel = self.system.create_wheel()
         wheel.setNode(wheel_np.node())
         wheel.setChassisConnectionPointCs(pos)
@@ -579,6 +654,8 @@ class BaseVehicle(BaseObject, BaseVehicleState):
 
     def add_image_sensor(self, name: str, sensor: ImageBuffer):
         self.image_sensors[name] = sensor
+        self.engine.graphicsEngine.render_frame()
+        self.engine.graphicsEngine.render_frame()
 
     def add_navigation(self):
         if not self.config["need_navigation"]:
@@ -587,18 +664,26 @@ class BaseVehicle(BaseObject, BaseVehicleState):
         if navi is None:
             navi = NodeNetworkNavigation if self.engine.current_map.road_network_type == NodeRoadNetwork \
                 else EdgeNetworkNavigation
-        self.navigation = \
-            navi(self.engine,
-                 show_navi_mark=self.engine.global_config["vehicle_config"]["show_navi_mark"],
-                 random_navi_mark_color=self.engine.global_config["vehicle_config"]["random_navi_mark_color"],
-                 show_dest_mark=self.engine.global_config["vehicle_config"]["show_dest_mark"],
-                 show_line_to_dest=self.engine.global_config["vehicle_config"]["show_line_to_dest"],
-                 panda_color=self.panda_color
-                 )
+        self.navigation = navi(
+            # self.engine,
+            show_navi_mark=self.engine.global_config["vehicle_config"]["show_navi_mark"],
+            random_navi_mark_color=self.engine.global_config["vehicle_config"]["random_navi_mark_color"],
+            show_dest_mark=self.engine.global_config["vehicle_config"]["show_dest_mark"],
+            show_line_to_dest=self.engine.global_config["vehicle_config"]["show_line_to_dest"],
+            panda_color=self.panda_color,
+            name=self.name,
+            vehicle_config=self.config
+        )
 
     def update_map_info(self, map):
         """
-        Update map info after reset()
+        Update map information that are used by this vehicle, after reset()
+        This function will query the map about the spawn position and destination of current vehicle,
+        and update the navigation module by feeding the information of spawn point and destination.
+
+        For the spawn position, if it is not specify in the config["spawn_lane_index"], we will automatically
+        select one lane based on the localization results.
+
         :param map: new map
         :return: None
         """
@@ -608,12 +693,18 @@ class BaseVehicle(BaseObject, BaseVehicleState):
             self.heading, self.spawn_place, self.engine, return_all_result=True, use_heading_filter=False
         )
         possible_lane_indexes = [lane_index for lane, lane_index, dist in possible_lanes]
-        try:
+
+        if len(possible_lanes) == 0 and self.config["spawn_lane_index"] is None:
+            from metadrive.utils.error_class import NavigationError
+            raise NavigationError("Can't find valid navigation for this car.")
+
+        if self.config["spawn_lane_index"] is not None and self.config["spawn_lane_index"] in possible_lane_indexes:
             idx = possible_lane_indexes.index(self.config["spawn_lane_index"])
-        except ValueError:
-            lane, new_l_index = possible_lanes[0][:-1]
-        else:
             lane, new_l_index = possible_lanes[idx][:-1]
+        else:
+            assert len(possible_lanes) > 0
+            lane, new_l_index = possible_lanes[0][:-1]
+
         dest = self.config["destination"]
         self.navigation.reset(
             map,
@@ -634,18 +725,27 @@ class BaseVehicle(BaseObject, BaseVehicleState):
         for contact in result_1.getContacts() + result_2.getContacts():
             node0 = contact.getNode0()
             node1 = contact.getNode1()
-            name = [node0.getName(), node1.getName()]
-            name.remove(BodyName.Vehicle)
-            if name[0] == BodyName.White_continuous_line:
+            node = node0 if node1.getName() == BodyName.Vehicle else node1
+            name = node.getName()
+            if name == BodyName.White_continuous_line:
                 self.on_white_continuous_line = True
-            elif name[0] == BodyName.Yellow_continuous_line:
+            elif name == BodyName.Yellow_continuous_line:
                 self.on_yellow_continuous_line = True
-            elif name[0] == BodyName.Broken_line:
+            elif name == BodyName.Broken_line:
                 self.on_broken_line = True
+            elif name == BodyName.TrafficLight:
+                light = get_object_from_node(node)
+                if light.status == TrafficLightStatus.GREEN:
+                    self.green_light = True
+                elif light.status == TrafficLightStatus.RED:
+                    self.red_light = True
+                elif light.status == TrafficLightStatus.YELLOW:
+                    self.yellow_light = True
+                name = TrafficLightStatus.semantics(light.status)
             else:
                 # didn't add
                 continue
-            contacts.add(name[0])
+            contacts.add(name)
         # side walk detect
         res = rect_region_detection(
             self.engine,
@@ -678,13 +778,13 @@ class BaseVehicle(BaseObject, BaseVehicleState):
             self.lidar = None
         if len(self.image_sensors) != 0:
             for sensor in self.image_sensors.values():
-                sensor.destroy()
+                if sensor is not self.engine.main_camera:
+                    sensor.destroy()
         self.image_sensors = {}
-        self.engine = None
 
     def set_heading_theta(self, heading_theta, rad_to_degree=True) -> None:
         """
-        Set heading theta for this object
+        Set heading theta for this object. Vehicle local frame has a 90 degree offset
         :param heading_theta: float in rad
         """
         h = panda_heading(heading_theta)
@@ -717,6 +817,31 @@ class BaseVehicle(BaseObject, BaseVehicleState):
             )
         return state
 
+    # def get_raw_state(self):
+    #     ret = dict(position=self.position, heading=self.heading, velocity=self.velocity)
+    #     return ret
+
+    def get_dynamics_parameters(self):
+        # These two can be changed on the fly
+        max_engine_force = self.config["max_engine_force"]
+        max_brake_force = self.config["max_brake_force"]
+
+        # These two can only be changed in init
+        wheel_friction = self.config["wheel_friction"]
+        assert self.max_steering == self.config["max_steering"]
+        max_steering = self.max_steering
+
+        mass = self.config["mass"] if self.config["mass"] else self.MASS
+
+        ret = dict(
+            max_engine_force=max_engine_force,
+            max_brake_force=max_brake_force,
+            wheel_friction=wheel_friction,
+            max_steering=max_steering,
+            mass=mass
+        )
+        return ret
+
     def set_state(self, state):
         super(BaseVehicle, self).set_state(state)
         self.throttle_brake = state["throttle_brake"]
@@ -742,33 +867,15 @@ class BaseVehicle(BaseObject, BaseVehicleState):
     def get_overtake_num(self):
         return len(self.front_vehicles.intersection(self.back_vehicles))
 
-    @classmethod
-    def get_action_space_before_init(
-        cls, extra_action_dim: int = 0, discrete_action=False, steering_dim=5, throttle_dim=5
-    ):
-        if not discrete_action:
-            return gym.spaces.Box(-1.0, 1.0, shape=(2 + extra_action_dim, ), dtype=np.float32)
-        else:
-            return gym.spaces.MultiDiscrete([steering_dim, throttle_dim])
-
     def __del__(self):
         super(BaseVehicle, self).__del__()
-        self.engine = None
+        # self.engine = None
         self.lidar = None
         self.mini_map = None
         self.rgb_camera = None
         self.navigation = None
         self.wheels = None
 
-    @property
-    def arrive_destination(self):
-        long, lat = self.navigation.final_lane.local_coordinates(self.position)
-        flag = (self.navigation.final_lane.length - 5 < long < self.navigation.final_lane.length + 5) and (
-            self.navigation.get_current_lane_width() / 2 >= lat >=
-            (0.5 - self.navigation.get_current_lane_num()) * self.navigation.get_current_lane_width()
-        )
-        return flag
-
     @property
     def reference_lanes(self):
         return self.navigation.current_ref_lanes
@@ -780,7 +887,7 @@ class BaseVehicle(BaseObject, BaseVehicleState):
 
     @property
     def overspeed(self):
-        return True if self.lane.speed_limit < self.speed else False
+        return True if self.lane.speed_limit < self.speed_km_h else False
 
     @property
     def replay_done(self):
@@ -851,13 +958,17 @@ class BaseVehicle(BaseObject, BaseVehicleState):
         theta = wrap_to_pi(self.heading_theta) + wrap_to_pi(theta)
         norm_len = norm(project_on_heading, project_on_side)
         position = self.position
-        heading = np.sin(theta) * norm_len
-        side = np.cos(theta) * norm_len
+        heading = math.sin(theta) * norm_len
+        side = math.cos(theta) * norm_len
         return position[0] + side, position[1] + heading
 
     @property
-    def max_speed(self):
-        return self.config["max_speed"]
+    def max_speed_km_h(self):
+        return self.config["max_speed_km_h"]
+
+    @property
+    def max_speed_m_s(self):
+        return self.config["max_speed_km_h"] / 3.6
 
     @property
     def top_down_length(self):
@@ -883,3 +994,22 @@ class BaseVehicle(BaseObject, BaseVehicleState):
             rand_c = color[2]  # A pretty green
             c = rand_c
         return c
+
+    def before_reset(self):
+        for obj in [self.navigation, self.lidar, self.side_detector, self.lane_line_detector]:
+            if obj is not None and hasattr(obj, "before_reset"):
+                obj.before_reset()
+
+    def show_coordinates(self):
+        if self.coordinates_debug_np is not None:
+            return
+        height = self.HEIGHT
+        self.coordinates_debug_np = NodePath("debug coordinate")
+        # 90 degree diff
+        x = self.engine.add_line([0, 0, height], [0, -2, height], [0, 1, 0, 1], 1)
+        y = self.engine.add_line([0, 0, height], [1, 0, height], [0, 1, 0, 1], 1)
+        z = self.engine.add_line([0, 0, height], [0, 0, height + 0.5], [0, 0, 1, 1], 2)
+        x.reparentTo(self.coordinates_debug_np)
+        y.reparentTo(self.coordinates_debug_np)
+        z.reparentTo(self.coordinates_debug_np)
+        self.coordinates_debug_np.reparentTo(self.origin)
