@@ -1,8 +1,8 @@
 from typing import Tuple, Union, List
 
 import numpy as np
-from metadrive.component.highway_vehicle.controller import ControlledVehicle
-from metadrive.component.highway_vehicle.kinematics import Vehicle
+from metadrive.component.vehicle_model.controller import ControlledVehicle
+from metadrive.component.vehicle_model.kinematics import Vehicle
 
 import metadrive.utils.math_utils as utils
 from metadrive.component.static_object.base_static_object import BaseStaticObject
