@@ -1,6 +1,6 @@
-import math
 from typing import Set
 
+import math
 import numpy as np
 from panda3d.bullet import BulletGhostNode, BulletCylinderShape
 from panda3d.core import NodePath
@@ -10,8 +10,8 @@ from metadrive.component.vehicle_module.distance_detector import DistanceDetecto
 from metadrive.constants import CamMask, CollisionGroup
 from metadrive.engine.engine_utils import get_engine
 from metadrive.utils.coordinates_shift import panda_position
-from metadrive.utils.utils import get_object_from_node
 from metadrive.utils.math_utils import norm, clip
+from metadrive.utils.utils import get_object_from_node
 
 
 class Lidar(DistanceDetector):
@@ -24,7 +24,7 @@ class Lidar(DistanceDetector):
     def __init__(self, num_lasers: int = 240, distance: float = 50, enable_show=False):
         super(Lidar, self).__init__(num_lasers, distance, enable_show)
         self.origin.hide(CamMask.RgbCam | CamMask.Shadow | CamMask.Shadow | CamMask.DepthCam)
-        self.mask = CollisionGroup.Vehicle | CollisionGroup.InvisibleWall | CollisionGroup.TrafficObject
+        self.mask = CollisionGroup.can_be_lidar_detected()
 
         # lidar can calculate the detector mask by itself
         self.angle_delta = 360 / num_lasers if num_lasers > 0 else None
@@ -36,6 +36,8 @@ class Lidar(DistanceDetector):
         engine.physics_world.static_world.attach(self.broad_detector.node())
         self.enable_mask = True if not engine.global_config["_disable_detector_mask"] else False
 
+        self._node_path_list.append(self.broad_detector)
+
     def perceive(self, base_vehicle, detector_mask=True):
         res = self._get_lidar_mask(base_vehicle)
         lidar_mask = res[0] if detector_mask and self.enable_mask else None
@@ -80,9 +82,9 @@ class Lidar(DistanceDetector):
                 res.append(clip((relative_position[0] / self.perceive_distance + 1) / 2, 0.0, 1.0))
                 res.append(clip((relative_position[1] / self.perceive_distance + 1) / 2, 0.0, 1.0))
 
-                relative_velocity = ego_vehicle.projection(vehicle.velocity - ego_vehicle.velocity)
-                res.append(clip((relative_velocity[0] / ego_vehicle.max_speed + 1) / 2, 0.0, 1.0))
-                res.append(clip((relative_velocity[1] / ego_vehicle.max_speed + 1) / 2, 0.0, 1.0))
+                relative_velocity = ego_vehicle.projection(vehicle.velocity_km_h - ego_vehicle.velocity_km_h)
+                res.append(clip((relative_velocity[0] / ego_vehicle.max_speed_km_h + 1) / 2, 0.0, 1.0))
+                res.append(clip((relative_velocity[1] / ego_vehicle.max_speed_km_h + 1) / 2, 0.0, 1.0))
 
                 if add_others_navi:
                     ckpt1, ckpt2 = vehicle.navigation.get_checkpoints()
@@ -108,7 +110,7 @@ class Lidar(DistanceDetector):
         pos1 = vehicle.position
         head1 = vehicle.heading_theta
 
-        mask = np.zeros((self.num_lasers, ), dtype=np.bool)
+        mask = np.zeros((self.num_lasers, ), dtype=bool)
         mask.fill(False)
         objs = self.get_surrounding_objects(vehicle)
         for obj in objs:
