@@ -1,11 +1,11 @@
-import math
 from typing import List, Tuple
 
+import math
 from panda3d.bullet import BulletWorld
 from panda3d.core import Vec3
 from panda3d.core import Vec4, BitMask32
 
-EDITION = "MetaDrive v0.2.6.0"
+EDITION = "MetaDrive v0.3.0.1"
 DEFAULT_AGENT = "default_agent"
 RENDER_MODE_NONE = "none"  # Do not render
 RENDER_MODE_ONSCREEN = "onscreen"  # Pop up a window and draw image in it
@@ -53,6 +53,24 @@ COLLISION_INFO_COLOR = dict(
 )
 
 
+class TrafficLightStatus:
+    GREEN = 1
+    RED = 2
+    YELLOW = 3
+    UNKNOWN = 4
+
+    @classmethod
+    def semantics(self, status):
+        if status == self.GREEN:
+            return "Traffic Light: Green"
+        if status == self.RED:
+            return "Traffic Light: Red"
+        if status == self.YELLOW:
+            return "Traffic Light: Yellow"
+        if status == self.UNKNOWN:
+            return "Traffic Light: Unknown"
+
+
 class BodyName:
     White_continuous_line = "White Continuous Line"
     Yellow_continuous_line = "Yellow Continuous Line"
@@ -60,21 +78,27 @@ class BodyName:
     Sidewalk = "Sidewalk"
     Ground = "Ground"
     InvisibleWall = "InvisibleWall"
+    TrafficLight = "Traffic Light"
     Vehicle = "Vehicle"
     Lane = "Lane"
     Traffic_object = "Traffic Object"
     TollGate = "Toll Gate"
+    Pedestrian = "Pedestrian"
+    Cyclist = "Cyclist"
 
 
 COLOR = {
     BodyName.Sidewalk: "red",
     BodyName.White_continuous_line: "orange",
-    BodyName.Yellow_continuous_line: "red",
+    BodyName.Yellow_continuous_line: "orange",
     BodyName.Broken_line: "yellow",
     BodyName.Vehicle: "red",
     BodyName.Traffic_object: "orange",
     BodyName.InvisibleWall: "red",
     BodyName.TollGate: "red",
+    TrafficLightStatus.semantics(TrafficLightStatus.RED): "red",
+    TrafficLightStatus.semantics(TrafficLightStatus.YELLOW): "orange",
+    TrafficLightStatus.semantics(TrafficLightStatus.GREEN): "yellow",
 }
 
 
@@ -123,9 +147,13 @@ class CollisionGroup(Mask):
     ContinuousLaneLine = BitMask32.bit(7)
     InvisibleWall = BitMask32.bit(8)
     LidarBroadDetector = BitMask32.bit(9)
+    TrafficParticipants = BitMask32.bit(10)
 
     @classmethod
     def collision_rules(cls):
+        """
+        This should be a diagonal matrix
+        """
         return [
             # terrain collision
             (cls.Terrain, cls.Terrain, False),
@@ -137,6 +165,7 @@ class CollisionGroup(Mask):
             (cls.Terrain, cls.Sidewalk, True),
             (cls.Terrain, cls.LidarBroadDetector, False),
             (cls.Terrain, cls.TrafficObject, True),
+            (cls.Terrain, cls.TrafficParticipants, True),
 
             # block collision
             (cls.BrokenLaneLine, cls.BrokenLaneLine, False),
@@ -148,6 +177,7 @@ class CollisionGroup(Mask):
             (cls.BrokenLaneLine, cls.Sidewalk, False),
             (cls.BrokenLaneLine, cls.LidarBroadDetector, False),
             (cls.BrokenLaneLine, cls.TrafficObject, True),
+            (cls.BrokenLaneLine, cls.TrafficParticipants, True),
 
             # ego vehicle collision
             (cls.Vehicle, cls.Vehicle, True),
@@ -157,6 +187,7 @@ class CollisionGroup(Mask):
             (cls.Vehicle, cls.Sidewalk, True),
             (cls.Vehicle, cls.LidarBroadDetector, True),
             (cls.Vehicle, cls.TrafficObject, True),
+            (cls.Vehicle, cls.TrafficParticipants, True),
 
             # lane surface
             (cls.LaneSurface, cls.LaneSurface, False),
@@ -165,6 +196,7 @@ class CollisionGroup(Mask):
             (cls.LaneSurface, cls.Sidewalk, False),
             (cls.LaneSurface, cls.LidarBroadDetector, False),
             (cls.LaneSurface, cls.TrafficObject, True),
+            (cls.LaneSurface, cls.TrafficParticipants, True),
 
             # continuous lane line
             (cls.ContinuousLaneLine, cls.ContinuousLaneLine, False),
@@ -172,21 +204,32 @@ class CollisionGroup(Mask):
             (cls.ContinuousLaneLine, cls.Sidewalk, False),
             (cls.ContinuousLaneLine, cls.LidarBroadDetector, False),
             (cls.ContinuousLaneLine, cls.TrafficObject, False),
+            (cls.ContinuousLaneLine, cls.TrafficParticipants, True),
 
             # invisible wall
             (cls.InvisibleWall, cls.InvisibleWall, False),
             (cls.InvisibleWall, cls.Sidewalk, False),
             (cls.InvisibleWall, cls.LidarBroadDetector, True),
             (cls.InvisibleWall, cls.TrafficObject, False),
+            (cls.InvisibleWall, cls.TrafficParticipants, True),
 
             # side walk
             (cls.Sidewalk, cls.Sidewalk, False),
             (cls.Sidewalk, cls.LidarBroadDetector, False),
             (cls.Sidewalk, cls.TrafficObject, True),
+            (cls.Sidewalk, cls.TrafficParticipants, True),
 
-            #
+            # LidarBroadDetector
             (cls.LidarBroadDetector, cls.LidarBroadDetector, False),
             (cls.LidarBroadDetector, cls.TrafficObject, True),
+            (cls.LidarBroadDetector, cls.TrafficParticipants, True),
+
+            # TrafficObject
+            (cls.TrafficObject, cls.TrafficObject, True),
+            (cls.TrafficObject, cls.TrafficParticipants, True),
+
+            # TrafficParticipant
+            (cls.TrafficParticipants, cls.TrafficParticipants, True)
         ]
 
     @classmethod
@@ -197,6 +240,10 @@ class CollisionGroup(Mask):
             relation = rule[-1]
             world.setGroupCollisionFlag(group_1, group_2, relation)
 
+    @classmethod
+    def can_be_lidar_detected(cls):
+        return cls.Vehicle | cls.InvisibleWall | cls.TrafficObject | cls.TrafficParticipants
+
 
 LaneIndex = Tuple[str, str, int]
 Route = List[LaneIndex]
@@ -225,7 +272,7 @@ class LineType:
 
 class LineColor:
     GREY = (1, 1, 1, 1)
-    YELLOW = (245 / 255, 192 / 255, 67 / 255, 1)
+    YELLOW = (255 / 255, 200 / 255, 0 / 255, 1)
 
 
 class DrivableAreaProperty:
@@ -239,7 +286,7 @@ class DrivableAreaProperty:
     LANE_SEGMENT_LENGTH = 4
     STRIPE_LENGTH = 1.5
     LANE_LINE_WIDTH = 0.15
-    LANE_LINE_THICKNESS = 0.01
+    LANE_LINE_THICKNESS = 0.016
 
     SIDEWALK_THICKNESS = 0.4
     SIDEWALK_LENGTH = 3
@@ -304,3 +351,7 @@ class WaymoLaneProperty:
     @staticmethod
     def get_line_type_and_line_color(waymo_type):
         pass
+
+
+class NuPlanLaneProperty:
+    pass
