@@ -3,8 +3,9 @@ import pickle
 import time
 from collections import OrderedDict
 from typing import Callable, Optional, Union, List, Dict, AnyStr
-from metadrive.base_class.base_object import BaseObject
+
 import numpy as np
+from panda3d.core import NodePath, Vec3
 
 from metadrive.base_class.randomizable import Randomizable
 from metadrive.engine.core.engine_core import EngineCore
@@ -60,18 +61,19 @@ class BaseEngine(EngineCore, Randomizable):
         # topdown renderer
         self._top_down_renderer = None
 
+        # lanes debug
+        self.lane_coordinates_debug_node = None
+
+        # for multi-thread rendering
+        self.graphicsEngine.renderFrame()
+        self.graphicsEngine.renderFrame()
+
     def add_policy(self, object_id, policy_class, *args, **kwargs):
         policy = policy_class(*args, **kwargs)
         self._object_policies[object_id] = policy
         if self.record_episode:
             assert self.record_manager is not None, "No record manager"
-            filtered_args = []
-            for arg in args:
-                filtered_args.append(arg) if not isinstance(arg, BaseObject) else filtered_args.append(BaseObject)
-            filtered_kwargs = {}
-            for k, v in kwargs.items():
-                filtered_kwargs[k] = v if not isinstance(arg, BaseObject) else BaseObject
-            self.record_manager.add_policy_info(object_id, policy_class, filtered_args, kwargs)
+            self.record_manager.add_policy_info(object_id, policy_class, *args, **kwargs)
         return policy
 
     def add_task(self, object_id, task):
@@ -180,13 +182,35 @@ class BaseEngine(EngineCore, Randomizable):
                 obj.destroy()
             else:
                 obj.detach_from_world(self.physics_world)
+
+                # We might want to remove some episode-relevant information when recycling some objects
+                if hasattr(obj, "before_reset"):
+                    obj.before_reset()
+
                 if obj.class_name not in self._dying_objects:
                     self._dying_objects[obj.class_name] = []
-                self._dying_objects[obj.class_name].append(obj)
+                # We have a limit for buffering objects
+                if len(self._dying_objects[obj.class_name]) < self.global_config["num_buffering_objects"]:
+                    self._dying_objects[obj.class_name].append(obj)
+                else:
+                    obj.destroy()
             if self.global_config["record_episode"] and not self.replay_episode:
                 self.record_manager.add_clear_info(obj)
         return exclude_objects.keys()
 
+    def clear_object_if_possible(self, obj, force_destroy):
+        if isinstance(obj, dict):
+            return
+        if obj in self._spawned_objects:
+            self.clear_objects([obj], force_destroy=force_destroy)
+        if force_destroy and \
+                obj.class_name in self._dying_objects and \
+                obj in self._dying_objects[obj.class_name]:
+            self._dying_objects[obj.class_name].remove(obj)
+            if hasattr(obj, "destroy"):
+                obj.destroy()
+        del obj
+
     def reset(self):
         """
         Clear and generate the whole scene
@@ -202,21 +226,53 @@ class BaseEngine(EngineCore, Randomizable):
         self.record_episode = self.global_config["record_episode"]
         self.only_reset_when_replay = self.global_config["only_reset_when_replay"]
 
+        _debug_memory_usage = False
+
+        if _debug_memory_usage:
+
+            def process_memory():
+                import psutil
+                import os
+                process = psutil.Process(os.getpid())
+                mem_info = process.memory_info()
+                return mem_info.rss
+
+            cm = process_memory()
+
         # reset manager
-        for manager in self._managers.values():
+        for manager_name, manager in self._managers.items():
             # clean all manager
             manager.before_reset()
+
+            if _debug_memory_usage:
+                lm = process_memory()
+                if lm - cm != 0:
+                    print("{}: Before Reset! Mem Change {:.3f}MB".format(manager_name, (lm - cm) / 1e6))
+                cm = lm
+
         self._object_clean_check()
 
-        for manager in self.managers.values():
+        for manager_name, manager in self.managers.items():
             if self.replay_episode and self.only_reset_when_replay and manager is not self.replay_manager:
                 # The scene will be generated from replay manager in only reset replay mode
                 continue
             manager.reset()
 
-        for manager in self.managers.values():
+            if _debug_memory_usage:
+                lm = process_memory()
+                if lm - cm != 0:
+                    print("{}: Reset! Mem Change {:.3f}MB".format(manager_name, (lm - cm) / 1e6))
+                cm = lm
+
+        for manager_name, manager in self.managers.items():
             manager.after_reset()
 
+            if _debug_memory_usage:
+                lm = process_memory()
+                if lm - cm != 0:
+                    print("{}: After Reset! Mem Change {:.3f}MB".format(manager_name, (lm - cm) / 1e6))
+                cm = lm
+
         # reset cam
         if self.main_camera is not None:
             self.main_camera.reset()
@@ -225,8 +281,8 @@ class BaseEngine(EngineCore, Randomizable):
                 current_track_vehicle = vehicles[0]
                 self.main_camera.set_follow_lane(self.global_config["use_chase_camera_follow_lane"])
                 self.main_camera.track(current_track_vehicle)
-                if self.global_config["is_multi_agent"]:
-                    self.main_camera.stop_track(bird_view_on_current_position=False)
+                # if self.global_config["is_multi_agent"]:
+                #     self.main_camera.stop_track(bird_view_on_current_position=False)
         self.taskMgr.step()
 
     def before_step(self, external_actions: Dict[AnyStr, np.array]):
@@ -311,7 +367,6 @@ class BaseEngine(EngineCore, Randomizable):
         if self.main_camera is not None:
             self.main_camera.destroy()
         self.interface.destroy()
-        self.clear_world()
         self.close_world()
 
         if self._top_down_renderer is not None:
@@ -352,7 +407,10 @@ class BaseEngine(EngineCore, Randomizable):
         if self.replay_episode:
             return self.replay_manager.current_map
         else:
-            return self.map_manager.current_map
+            if hasattr(self, "map_manager"):
+                return self.map_manager.current_map
+            else:
+                return None
 
     @property
     def current_track_vehicle(self):
@@ -371,8 +429,9 @@ class BaseEngine(EngineCore, Randomizable):
             return self.replay_manager.replay_agents
 
     def setup_main_camera(self):
-        from metadrive.engine.core.chase_camera import MainCamera
-        if self.global_config["use_render"]:
+        from metadrive.engine.core.main_camera import MainCamera
+        # Not we should always enable main camera if image obs is required! Or RGBCamera will return incorrect result
+        if self.global_config["use_render"] or self.global_config["image_observation"]:
             return MainCamera(self, self.global_config["camera_height"], self.global_config["camera_dist"])
         else:
             return None
@@ -446,3 +505,49 @@ class BaseEngine(EngineCore, Randomizable):
             from metadrive.obs.top_down_renderer import TopDownRenderer
             self._top_down_renderer = TopDownRenderer(*args, **kwargs)
         return self._top_down_renderer.render(text, *args, **kwargs)
+
+    def get_window_image(self, return_bytes=False):
+        window_count = self.graphicsEngine.getNumWindows() - 1
+        texture = self.graphicsEngine.getWindow(window_count).getDisplayRegion(0).getScreenshot()
+
+        assert texture.getXSize() == self.global_config["window_size"][0], (
+            texture.getXSize(), texture.getYSize(), self.global_config["window_size"]
+        )
+        assert texture.getYSize() == self.global_config["window_size"][1], (
+            texture.getXSize(), texture.getYSize(), self.global_config["window_size"]
+        )
+
+        image_bytes = texture.getRamImage().getData()
+
+        if return_bytes:
+            return image_bytes, (texture.getXSize(), texture.getYSize())
+
+        img = np.frombuffer(image_bytes, dtype=np.uint8)
+        img = img.reshape((texture.getYSize(), texture.getXSize(), 4))
+        img = img[::-1]  # Flip vertically
+        img = img[..., :-1]  # Discard useless alpha channel
+        img = img[..., ::-1]  # Correct the colors
+
+        return img
+
+    def show_lane_coordinates(self, lanes):
+        if self.lane_coordinates_debug_node is not None:
+            self.lane_coordinates_debug_node.detachNode()
+            self.lane_coordinates_debug_node.removeNode()
+
+        self.lane_coordinates_debug_node = NodePath("Lane Coordinates debug")
+        for lane in lanes:
+            long_start = lateral_start = lane.position(0, 0)
+            lateral_end = lane.position(0, 2)
+
+            long_end = long_start + lane.heading_at(0) * 4
+            np_x = self.add_line(Vec3(*long_start, 0), Vec3(*long_end, 0), color=[1, 0, 0, 1], thickness=2)
+            np_y = self.add_line(Vec3(*lateral_start, 0), Vec3(*lateral_end, 0), color=[0, 1, 0, 1], thickness=2)
+            np_x.reparentTo(self.lane_coordinates_debug_node)
+            np_y.reparentTo(self.lane_coordinates_debug_node)
+        self.lane_coordinates_debug_node.reparentTo(self.worldNP)
+
+    def remove_show_lane_coordinates(self):
+        if self.lane_coordinates_debug_node is not None:
+            self.lane_coordinates_debug_node.detachNode()
+            self.lane_coordinates_debug_node.removeNode()
