@@ -62,12 +62,14 @@ def attach_logo(engine):
 
 class EngineCore(ShowBase.ShowBase):
     DEBUG = False
+    global_config = None  # global config can exist before engine initialization
     loadPrcFileData("", "window-title {}".format(EDITION))
     loadPrcFileData("", "framebuffer-multisample 1")
     loadPrcFileData("", "multisamples 8")
     loadPrcFileData("", 'bullet-filter-algorithm groups-mask')
     loadPrcFileData("", "audio-library-name null")
     loadPrcFileData("", "model-cache-compressed-textures 1")
+    loadPrcFileData("", "textures-power-2 none")
 
     # loadPrcFileData("", "transform-cache 0")
     # loadPrcFileData("", "state-cache 0")
@@ -83,33 +85,51 @@ class EngineCore(ShowBase.ShowBase):
     # loadPrcFileData("", "gl-version 3 2")
 
     def __init__(self, global_config):
-        self.global_config = global_config
+        # if EngineCore.global_config is not None:
+        #     # assert global_config is EngineCore.global_config, \
+        #     #     "No allowed to change ptr of global config, which may cause issue"
+        #     pass
+        # else:
+        EngineCore.global_config = global_config
+
         if self.global_config["pstats"]:
             # pstats debug provided by panda3d
             loadPrcFileData("", "want-pstats 1")
 
-        loadPrcFileData("", "win-size {} {}".format(*self.global_config["window_size"]))
-
         # Setup onscreen render
         if self.global_config["use_render"]:
             self.mode = RENDER_MODE_ONSCREEN
             # Warning it may cause memory leak, Pand3d Official has fixed this in their master branch.
             # You can enable it if your panda version is latest.
-            loadPrcFileData("", "threading-model Cull/Draw")  # multi-thread render, accelerate simulation when evaluate
+            if self.global_config["multi_thread_render"]:
+                # multi-thread render, accelerate simulation
+                loadPrcFileData("", "threading-model {}".format(self.global_config["multi_thread_render_mode"]))
         else:
-            if self.global_config["offscreen_render"]:
+            self.global_config["show_coordinates"] = False
+            if self.global_config["image_observation"]:
                 self.mode = RENDER_MODE_OFFSCREEN
-                loadPrcFileData("", "threading-model Cull/Draw")
+                if self.global_config["multi_thread_render"]:
+                    loadPrcFileData("", "threading-model {}".format(self.global_config["multi_thread_render_mode"]))
+                if self.global_config["vehicle_config"]["image_source"] != "main_camera":
+                    # reduce size as we don't use the main camera content for improving efficiency
+                    self.global_config["window_size"] = (1, 1)
+
             else:
                 self.mode = RENDER_MODE_NONE
+                if self.global_config["show_interface"]:
+                    # Disable useless camera capturing in none mode
+                    self.global_config["show_interface"] = False
 
         if is_mac() and (self.mode == RENDER_MODE_OFFSCREEN):  # Mac don't support offscreen rendering
             self.mode = RENDER_MODE_ONSCREEN
 
+        loadPrcFileData("", "win-size {} {}".format(*self.global_config["window_size"]))
         # Setup some debug options
-        if self.global_config["headless_machine_render"]:
-            # headless machine support
-            loadPrcFileData("", "load-display  pandagles2")
+        # if self.global_config["headless_machine_render"]:
+        #     # headless machine support
+        #     # loadPrcFileData("", "load-display  pandagles2")
+        #     # no further actions will be applied now!
+        #     pass
         if self.global_config["debug"]:
             # debug setting
             EngineCore.DEBUG = True
@@ -195,6 +215,7 @@ class EngineCore(ShowBase.ShowBase):
         # some render attribute
         self.pbrpipe = None
         self.world_light = None
+        self.common_filter = None
 
         # physics world
         self.physics_world = PhysicsWorld(self.global_config["debug_static_world"])
@@ -217,7 +238,7 @@ class EngineCore(ShowBase.ShowBase):
                 render_node=None,
                 window=None,
                 camera_node=None,
-                msaa_samples=4,
+                msaa_samples=16,
                 max_lights=8,
                 use_normal_maps=False,
                 use_emission_maps=True,
@@ -229,7 +250,12 @@ class EngineCore(ShowBase.ShowBase):
             self.pbrpipe.render_node = self.pbr_render
             self.pbrpipe.render_node.set_antialias(AntialiasAttrib.M_auto)
             self.pbrpipe._recompile_pbr()
-            self.pbrpipe.manager.cleanup()
+            # self.pbrpipe.manager.cleanup()
+            #
+            # # filter
+            # from direct.filter.CommonFilters import CommonFilters
+            # self.common_filter = CommonFilters(self.win, self.cam)
+            # self.common_filter.set_gamma_adjust(0.8)
 
             # set main cam
             self.cam.node().setCameraMask(CamMask.MainCam)
@@ -267,6 +293,10 @@ class EngineCore(ShowBase.ShowBase):
         # task manager
         self.taskMgr.remove('audioLoop')
 
+        self.coordinate_line = []
+        if self.global_config["show_coordinates"]:
+            self.show_coordinates()
+
     def render_frame(self, text: Optional[Union[dict, str]] = None):
         """
         The real rendering is conducted by the igLoop task maintained by panda3d.
@@ -294,6 +324,7 @@ class EngineCore(ShowBase.ShowBase):
         debugNP = self.render.attachNewNode(debugNode)
         self.physics_world.dynamic_world.setDebugNode(debugNP.node())
         self.debug_node = debugNP
+        # TODO: debugNP is not registered.
 
     def toggleAnalyze(self):
         self.worldNP.analyze()
@@ -331,6 +362,7 @@ class EngineCore(ShowBase.ShowBase):
         self.physics_world.destroy()
         self.destroy()
         close_asset_loader()
+        EngineCore.global_config = None
 
         import sys
         if sys.version_info >= (3, 0):
@@ -363,6 +395,7 @@ class EngineCore(ShowBase.ShowBase):
         line_seg.drawTo(end_p[0] * self.w_scale, 0, end_p[1] * self.h_scale)
         line_seg.setThickness(thickness)
         line_np = self.aspect2d.attachNewNode(line_seg.create(False))
+        # TODO: line_np is not registered.
         return line_np
 
     def remove_logo(self, task):
@@ -376,12 +409,42 @@ class EngineCore(ShowBase.ShowBase):
             return task.cont
 
     def add_line(self, start_p: Union[Vec3, Tuple], end_p: Union[Vec3, Tuple], color, thickness: float):
+        assert self.mode == RENDER_MODE_ONSCREEN, "Can not call this API in render mode: {}".format(self.mode)
+        start_p = [*start_p]
+        end_p = [*end_p]
+        start_p[1] *= -1
+        end_p[1] *= -1
         line_seg = LineSegs("interface")
         line_seg.setColor(*color)
-        line_seg.moveTo(start_p)
-        line_seg.drawTo(end_p)
+        line_seg.moveTo(Vec3(*start_p))
+        line_seg.drawTo(Vec3(*end_p))
         line_seg.setThickness(thickness)
-        NodePath(line_seg.create(False)).reparentTo(self.render)
+        np = NodePath(line_seg.create(False))
+        # np.reparentTo(self.render)
+        return np
+
+    def show_coordinates(self):
+        if len(self.coordinate_line) > 0:
+            return
+        # x direction = red
+        np_x = self.add_line(Vec3(0, 0, 0), Vec3(100, 0, 0), color=[1, 0, 0, 1], thickness=2)
+        np_x.reparentTo(self.render)
+        # y direction = blue
+        np_y = self.add_line(Vec3(0, 0, 0), Vec3(0, 50, 0), color=[0, 1, 0, 1], thickness=2)
+        np_y.reparentTo(self.render)
+        self.coordinate_line.append(np_x)
+        self.coordinate_line.append(np_y)
+
+    def remove_coordinates(self):
+        for line in self.coordinate_line:
+            line.detachNode()
+            line.removeNode()
+
+    def set_coordinates_indicator_pos(self, pos):
+        if len(self.coordinate_line) == 0:
+            return
+        for line in self.coordinate_line:
+            line.setPos(pos[0], -pos[1], 0)
 
 
 if __name__ == "__main__":
