@@ -1,10 +1,13 @@
 import logging
+from direct.filter.FilterManager import FilterManager
+import panda3d.core as p3d
+from simplepbr import _load_shader_str
 from typing import Union, List
 
 import numpy as np
-from panda3d.core import NodePath, Vec3, Vec4, Camera, PNMImage
+from panda3d.core import NodePath, Vec3, Vec4, Camera, PNMImage, Shader, RenderState, ShaderAttrib
 
-from metadrive.constants import RENDER_MODE_ONSCREEN, BKG_COLOR
+from metadrive.constants import RENDER_MODE_ONSCREEN, BKG_COLOR, RENDER_MODE_NONE
 
 
 class ImageBuffer:
@@ -13,26 +16,30 @@ class ImageBuffer:
     BUFFER_W = 84  # left to right
     BUFFER_H = 84  # bottom to top
     BKG_COLOR = BKG_COLOR
-    display_bottom = 0.8
-    display_top = 1
+    # display_bottom = 0.8
+    # display_top = 1
     display_region = None
     display_region_size = [1 / 3, 2 / 3, 0.8, 1.0]
     line_borders = []
 
     def __init__(
         self,
-        length: float,
         width: float,
+        height: float,
         pos: Vec3,
         bkg_color: Union[Vec4, Vec3],
         parent_node: NodePath = None,
         frame_buffer_property=None,
+        setup_pbr=False,
         # engine=None
     ):
+
+        self._node_path_list = []
+
         # from metadrive.engine.engine_utils import get_engine
         # self.engine = engine or get_engine()
         try:
-            assert self.engine.win is not None, "{} cannot be made without use_render or offscreen_render".format(
+            assert self.engine.win is not None, "{} cannot be made without use_render or image_observation".format(
                 self.__class__.__name__
             )
             assert self.CAM_MASK is not None, "Define a camera mask for every image buffer"
@@ -40,18 +47,16 @@ class ImageBuffer:
             logging.debug("Cannot create {}".format(self.__class__.__name__))
             self.buffer = None
             self.cam = NodePath(Camera("non-sense camera"))
+            self._node_path_list.append(self.cam)
+
             self.lens = self.cam.node().getLens()
             return
 
-        if length > 100 or width > 100:
-            # Too large width or length will cause corruption in Mac.
-            logging.warning("You may using too large buffer! The width is {}, and length is {}.".format(width, length))
-
         # self.texture = Texture()
         if frame_buffer_property is None:
-            self.buffer = self.engine.win.makeTextureBuffer("camera", length, width)
+            self.buffer = self.engine.win.makeTextureBuffer("camera", width, height)
         else:
-            self.buffer = self.engine.win.makeTextureBuffer("camera", length, width, fbp=frame_buffer_property)
+            self.buffer = self.engine.win.makeTextureBuffer("camera", width, height, fbp=frame_buffer_property)
             # now we have to setup a new scene graph to make this scene
 
         self.origin = NodePath("new render")
@@ -63,6 +68,32 @@ class ImageBuffer:
         self.cam.node().setCameraMask(self.CAM_MASK)
         if parent_node is not None:
             self.origin.reparentTo(parent_node)
+        self.scene_tex = None
+        if setup_pbr:
+            self.manager = FilterManager(self.buffer, self.cam)
+            fbprops = p3d.FrameBufferProperties()
+            fbprops.float_color = True
+            fbprops.set_rgba_bits(16, 16, 16, 16)
+            fbprops.set_depth_bits(24)
+            fbprops.set_multisamples(self.engine.pbrpipe.msaa_samples)
+            self.scene_tex = p3d.Texture()
+            self.scene_tex.set_format(p3d.Texture.F_rgba16)
+            self.scene_tex.set_component_type(p3d.Texture.T_float)
+            self.tonemap_quad = self.manager.render_scene_into(colortex=self.scene_tex, fbprops=fbprops)
+            #
+            defines = {}
+            #
+            post_vert_str = _load_shader_str('post.vert', defines)
+            post_frag_str = _load_shader_str('tonemap.frag', defines)
+            tonemap_shader = p3d.Shader.make(
+                p3d.Shader.SL_GLSL,
+                vertex=post_vert_str,
+                fragment=post_frag_str,
+            )
+            self.tonemap_quad.set_shader(tonemap_shader)
+            self.tonemap_quad.set_shader_input('tex', self.scene_tex)
+            self.tonemap_quad.set_shader_input('exposure', 1.0)
+
         logging.debug("Load Image Buffer: {}".format(self.__class__.__name__))
 
     @property
@@ -76,7 +107,7 @@ class ImageBuffer:
         """
         # self.engine.graphicsEngine.renderFrame()
         img = PNMImage()
-        self.buffer.getScreenshot(img)
+        self.buffer.getDisplayRegions()[1].getScreenshot(img)
         return img
 
     def save_image(self, name="debug.png"):
@@ -89,12 +120,13 @@ class ImageBuffer:
     def get_rgb_array(self):
         if self.engine.episode_step <= 1:
             self.engine.graphicsEngine.renderFrame()
-        origin_img = self.cam.node().getDisplayRegion(0).getScreenshot()
-        v = memoryview(origin_img.getRamImage()).tolist()
-        img = np.array(v, dtype=np.uint8)
+        origin_img = self.buffer.getDisplayRegion(1).getScreenshot()
+        img = np.frombuffer(origin_img.getRamImage().getData(), dtype=np.uint8)
         img = img.reshape((origin_img.getYSize(), origin_img.getXSize(), 4))
+        # img = np.swapaxes(img, 1, 0)
         img = img[::-1]
-        return img[..., :-1]
+        img = img[..., :-1]
+        return img
 
     @staticmethod
     def get_grayscale_array(img, clip=True):
@@ -110,10 +142,10 @@ class ImageBuffer:
             return np.clip(numpy_array, 0, 1)
 
     def add_display_region(self, display_region: List[float]):
-        if self.engine.mode == RENDER_MODE_ONSCREEN and self.display_region is None:
+        if self.engine.mode != RENDER_MODE_NONE and self.display_region is None:
             # only show them when onscreen
             self.display_region = self.engine.win.makeDisplayRegion(*display_region)
-            self.display_region.setCamera(self.cam)
+            self.display_region.setCamera(self.buffer.getDisplayRegions()[1].camera)
             self.draw_border(display_region)
 
     def draw_border(self, display_region):
@@ -156,5 +188,12 @@ class ImageBuffer:
         if hasattr(self, "origin"):
             self.origin.removeNode()
 
+        from metadrive.base_class.base_object import clear_node_list
+        clear_node_list(self._node_path_list)
+
     def __del__(self):
         logging.debug("{} is destroyed".format(self.__class__.__name__))
+
+    @classmethod
+    def update_display_region_size(cls, display_region_size):
+        cls.display_region_size = display_region_size
