@@ -2,7 +2,7 @@
 import math
 
 from panda3d.bullet import BulletRigidBodyNode, BulletPlaneShape
-from panda3d.core import Vec3, CardMaker, LQuaternionf, TextureStage, Texture, SamplerState
+from panda3d.core import Vec3, CardMaker, TextureStage, Texture, LQuaternionf, SamplerState
 
 from metadrive.base_class.base_object import BaseObject
 from metadrive.constants import BodyName, CamMask, CollisionGroup
@@ -23,13 +23,16 @@ class Terrain(BaseObject):
         node.setIntoCollideMask(self.COLLISION_MASK)
         self.dynamic_nodes.append(node)
 
-        self.origin.attachNewNode(node)
+        np = self.origin.attachNewNode(node)
+        self._node_path_list.append(np)
+
         if self.render and show_terrain:
             self.origin.hide(CamMask.MiniMap | CamMask.Shadow | CamMask.DepthCam | CamMask.ScreenshotCam)
             # self.terrain_normal = self.loader.loadTexture(
             #     AssetLoader.file_path( "textures", "grass2", "normal.jpg")
             # )
             self.terrain_texture = self.loader.loadTexture(AssetLoader.file_path("textures", "ground.png"))
+            self.terrain_texture.set_format(Texture.F_srgb)
             self.terrain_texture.setWrapU(Texture.WM_repeat)
             self.terrain_texture.setWrapV(Texture.WM_repeat)
             self.ts_color = TextureStage("color")
@@ -40,7 +43,9 @@ class Terrain(BaseObject):
             scale = 20000
             cm.setUvRange((0, 0), (scale / 10, scale / 10))
             card = self.origin.attachNewNode(cm.generate())
-            # scale = 1 if self.use_hollow else 20000
+
+            self._node_path_list.append(card)
+
             card.set_scale(scale)
             card.setPos(-scale / 2, -scale / 2, -0.1)
             card.setZ(-.05)
@@ -49,3 +54,20 @@ class Terrain(BaseObject):
             self.terrain_texture.setMinfilter(SamplerState.FT_linear_mipmap_linear)
             self.terrain_texture.setAnisotropicDegree(8)
             card.setQuat(LQuaternionf(math.cos(-math.pi / 4), math.sin(-math.pi / 4), 0, 0))
+
+            # self.GROUND = GeoMipTerrain("mySimpleTerrain")
+            # self.GROUND.setHeightfield(AssetLoader.file_path("textures", "height_map.png"))
+            # self.GROUND.setAutoFlatten(GeoMipTerrain.AFMStrong)
+            # # terrain.setBruteforce(True)
+            # # # Since the terrain is a texture, shader will not calculate the depth information, we add a moving terrain
+            # # # model to enable the depth information of terrain
+            # self.GROUND_MODEL = self.GROUND.getRoot()
+            # self.GROUND_MODEL.setPos(-128, -128, -0.5)
+            # self.GROUND_MODEL.reparentTo(self.origin)
+            # self.GROUND_MODEL.setTexture(self.ts_color, self.terrain_texture)
+            # self.GROUND_MODEL.setTexture(self.ts_normal, self.terrain_normal)
+            # self.GROUND_MODEL.setTexScale(self.ts_color, 100, 100)
+            # self.GROUND_MODEL.setTexScale(self.ts_normal, 100, 100)
+            # # self.GROUND_MODEL.hide(CamMask.AllOn)
+            # # self.GROUND_MODEL.show(CamMask.DepthCam)
+            # self.GROUND.generate()
