@@ -1,28 +1,31 @@
 import time
+
 from collections import defaultdict
 from typing import Union, Dict, AnyStr, Optional, Tuple
-from metadrive.manager.record_manager import RecordManager
-from metadrive.manager.replay_manager import ReplayManager
+from metadrive.component.vehicle_module.mini_map import MiniMap
+from metadrive.component.vehicle_module.rgb_camera import RGBCamera
+from metadrive.component.vehicle_module.vehicle_panel import VehiclePanel
 import gym
 import numpy as np
 from panda3d.core import PNMImage
 
-from metadrive.component.vehicle.base_vehicle import BaseVehicle
-from metadrive.constants import RENDER_MODE_NONE, DEFAULT_AGENT, REPLAY_DONE
-from metadrive.engine.base_engine import BaseEngine
+from metadrive.constants import RENDER_MODE_NONE, DEFAULT_AGENT
 from metadrive.engine.engine_utils import initialize_engine, close_engine, \
-    engine_initialized, set_global_random_seed
+    engine_initialized, set_global_random_seed, initialize_global_config, get_global_config
 from metadrive.manager.agent_manager import AgentManager
+from metadrive.manager.record_manager import RecordManager
+from metadrive.manager.replay_manager import ReplayManager
 from metadrive.obs.image_obs import ImageStateObservation
 from metadrive.obs.observation_base import ObservationBase
 from metadrive.obs.state_obs import LidarStateObservation
+from metadrive.policy.env_input_policy import EnvInputPolicy
 from metadrive.utils import Config, merge_dicts, get_np_random, concat_step_infos
 
 BASE_DEFAULT_CONFIG = dict(
 
     # ===== agent =====
     random_agent_model=False,
-    agent_policy=None,
+    agent_policy=EnvInputPolicy,
 
     # ===== multi-agent =====
     num_agents=1,  # Note that this can be set to >1 in MARL envs, or set to -1 for as many vehicles as possible.
@@ -39,6 +42,9 @@ BASE_DEFAULT_CONFIG = dict(
     discrete_action=False,
     discrete_steering_dim=5,
     discrete_throttle_dim=5,
+    # When discrete_action=True: If True, use MultiDiscrete action space. Otherwise, use Discrete.
+    use_multi_discrete=False,
+    action_check=False,
 
     # ===== Rendering =====
     use_render=False,  # pop a window to render or not
@@ -47,7 +53,9 @@ BASE_DEFAULT_CONFIG = dict(
     cull_scene=True,  # only for debug use
     use_chase_camera_follow_lane=False,  # If true, then vision would be more stable.
     camera_height=1.8,
-    camera_dist=6.0,
+    camera_dist=6.5,
+    camera_pitch=None,  # degree
+    camera_smooth=True,  # degree
     prefer_track_agent=None,
     draw_map_resolution=1024,  # Drawing the map in a canvas of (x, x) pixels.
     top_down_camera_initial_x=0,
@@ -65,10 +73,11 @@ BASE_DEFAULT_CONFIG = dict(
         random_navi_mark_color=False,
         show_dest_mark=False,
         show_line_to_dest=False,
+        show_line_to_navi_mark=False,
         use_special_color=False,
 
         # ===== use image =====
-        image_source="rgb_camera",  # take effect when only when offscreen_render == True
+        image_source="rgb_camera",  # take effect when only when image_observation == True
 
         # ===== vehicle spawn and destination =====
         navigation_module=None,  # a class type for self-defined navigation
@@ -78,12 +87,19 @@ BASE_DEFAULT_CONFIG = dict(
         spawn_lateral=0.0,
         destination=None,
         spawn_position_heading=None,
+        spawn_velocity=None,  # m/s
+        spawn_velocity_car_frame=False,
 
         # ==== others ====
         overtake_stat=False,  # we usually set to True when evaluation
-        action_check=False,
         random_color=False,
         random_agent_model=False,  # this will be overwritten by env.config["random_agent_model"]
+        # The shape of vehicle are predefined by its class. But in special case (WaymoVehicle) we might want to
+        # set to arbitrary shape.
+        width=None,
+        length=None,
+        height=None,
+        mass=None,
 
         # ===== vehicle module config =====
         lidar=dict(
@@ -94,7 +110,8 @@ BASE_DEFAULT_CONFIG = dict(
         show_lidar=False,
         mini_map=(84, 84, 250),  # buffer length, width
         rgb_camera=(84, 84),  # buffer length, width
-        depth_camera=(84, 84, True),  # buffer length, width, view_ground
+        depth_camera=(84, 84, False),  # buffer length, width, view_ground
+        main_camera=None,  # buffer length, width
         show_side_detector=False,
         show_lane_line_detector=False,
 
@@ -113,17 +130,18 @@ BASE_DEFAULT_CONFIG = dict(
     window_size=(1200, 900),  # or (width, height), if set to None, it will be automatically determined
     physics_world_step_size=2e-2,
     show_fps=True,
-    global_light=False,
+    global_light=True,
     # only render physics world without model, a special debug option
     debug_physics_world=False,
     # debug static world
     debug_static_world=False,
-    # set to true only when on headless machine and use rgb image!!!!!!
-    headless_machine_render=False,
+    # (Deprecated) set to true only when on headless machine and use rgb image!!!!!!
     # turn on to profile the efficiency
     pstats=False,
     # if need running in offscreen
-    offscreen_render=False,
+    image_observation=False,
+    # this is an advanced feature for accessing image with moving them to ram!
+    image_on_cuda=False,
     # accelerate the lidar perception
     _disable_detector_mask=False,
     # clip rgb to (0, 1)
@@ -132,6 +150,11 @@ BASE_DEFAULT_CONFIG = dict(
     force_render_fps=None,
     # if set to True all objects will be force destroyed when call clear()
     force_destroy=False,
+    # number of buffering objects for each class.
+    # we will maintain a set of buffers in the engine to store the used objects and can reuse them
+    # when possible. But it is possible that some classes of objects are always forcefully respawn
+    # and thus those used objects are stored in the buffer and never be reused.
+    num_buffering_objects=200,
 
     # ===== Others =====
     # The maximum distance used in PGLOD. Set to None will use the default values.
@@ -145,6 +168,10 @@ BASE_DEFAULT_CONFIG = dict(
     show_skybox=True,
     show_terrain=True,
     show_interface=True,
+    interface_panel=[MiniMap, RGBCamera, VehiclePanel],
+    show_coordinates=False,  # show coordinates for maps and objects for debug
+    multi_thread_render=True,
+    multi_thread_render_mode="Cull",  # or "Cull/Draw"
 
     # record/replay metadata
     record_episode=False,  # when replay_episode is not None ,this option will be useless
@@ -168,13 +195,18 @@ class BaseEnv(gym.Env):
             config = {}
         merged_config = self._merge_extra_config(config)
         global_config = self._post_process_config(merged_config)
+        global_config["vehicle_config"]["main_camera"] = global_config["window_size"]
+
         self.config = global_config
+        initialize_global_config(self.config)
 
         # agent check
         self.num_agents = self.config["num_agents"]
         self.is_multi_agent = self.config["is_multi_agent"]
         if not self.is_multi_agent:
             assert self.num_agents == 1
+        else:
+            assert not self.config["image_on_cuda"], "Image on cuda don't support Multi-agent!"
         assert isinstance(self.num_agents, int) and (self.num_agents > 0 or self.num_agents == -1)
 
         # observation and action space
@@ -183,7 +215,7 @@ class BaseEnv(gym.Env):
         )
 
         # lazy initialization, create the main vehicle in the lazy_init() func
-        self.engine: Optional[BaseEngine] = None
+        # self.engine: Optional[BaseEngine] = None
 
         # In MARL envs with respawn mechanism, varying episode lengths might happen.
         self.dones = None
@@ -209,19 +241,11 @@ class BaseEnv(gym.Env):
     def _get_action_space(self):
         if self.is_multi_agent:
             return {
-                v_id: BaseVehicle.get_action_space_before_init(
-                    self.config["vehicle_config"]["extra_action_dim"], self.config["discrete_action"],
-                    self.config["discrete_steering_dim"], self.config["discrete_throttle_dim"]
-                )
+                v_id: self.config["agent_policy"].get_input_space()
                 for v_id in self.config["target_vehicle_configs"].keys()
             }
         else:
-            return {
-                DEFAULT_AGENT: BaseVehicle.get_action_space_before_init(
-                    self.config["vehicle_config"]["extra_action_dim"], self.config["discrete_action"],
-                    self.config["discrete_steering_dim"], self.config["discrete_throttle_dim"]
-                )
-            }
+            return {DEFAULT_AGENT: self.config["agent_policy"].get_input_space()}
 
     def lazy_init(self):
         """
@@ -231,28 +255,33 @@ class BaseEnv(gym.Env):
         # It is the true init() func to create the main vehicle and its module, to avoid incompatible with ray
         if engine_initialized():
             return
-        self.engine = initialize_engine(self.config)
+        engine = initialize_engine(self.config)
         # engine setup
         self.setup_engine()
         # other optional initialization
         self._after_lazy_init()
 
+    @property
+    def engine(self):
+        from metadrive.engine.engine_utils import get_engine
+        return get_engine()
+
     def _after_lazy_init(self):
         pass
 
     # ===== Run-time =====
-    def step(self, actions: Union[np.ndarray, Dict[AnyStr, np.ndarray]]):
+    def step(self, actions: Union[np.ndarray, Dict[AnyStr, np.ndarray], int]):
         actions = self._preprocess_actions(actions)
         engine_info = self._step_simulator(actions)
         o, r, d, i = self._get_step_return(actions, engine_info=engine_info)
         return o, r, d, i
 
-    def _preprocess_actions(self, actions: Union[np.ndarray, Dict[AnyStr, np.ndarray]]) \
-            -> Union[np.ndarray, Dict[AnyStr, np.ndarray]]:
+    def _preprocess_actions(self, actions: Union[np.ndarray, Dict[AnyStr, np.ndarray], int]) \
+            -> Union[np.ndarray, Dict[AnyStr, np.ndarray], int]:
         if not self.is_multi_agent:
             actions = {v_id: actions for v_id in self.vehicles.keys()}
         else:
-            if self.config["vehicle_config"]["action_check"]:
+            if self.config["action_check"]:
                 # Check whether some actions are not provided.
                 given_keys = set(actions.keys())
                 have_keys = set(self.vehicles.keys())
@@ -292,7 +321,12 @@ class BaseEnv(gym.Env):
     def done_function(self, vehicle_id: str) -> Tuple[bool, Dict]:
         raise NotImplementedError()
 
-    def render(self, mode='human', text: Optional[Union[dict, str]] = None, *args, **kwargs) -> Optional[np.ndarray]:
+    def render(self,
+               mode='human',
+               text: Optional[Union[dict, str]] = None,
+               return_bytes=False,
+               *args,
+               **kwargs) -> Optional[np.ndarray]:
         """
         This is a pseudo-render function, only used to update onscreen message when using panda3d backend
         :param mode: 'rgb'/'human'
@@ -303,24 +337,28 @@ class BaseEnv(gym.Env):
             ret = self._render_topdown(text=text, *args, **kwargs)
             return ret
         assert self.config["use_render"] or self.engine.mode != RENDER_MODE_NONE, \
-            ("Panda Renderring is off now, can not render")
+            ("Panda Renderring is off now, can not render. Please set config['use_render'] = True!")
+
         self.engine.render_frame(text)
-        if mode != "human" and self.config["offscreen_render"]:
+
+        if mode != "human" and self.config["image_observation"]:
             # fetch img from img stack to be make this func compatible with other render func in RL setting
             return self.vehicle.observations.img_obs.get_image()
 
-        if mode == "rgb_array" and self.config["use_render"]:
-            if not hasattr(self, "_temporary_img_obs"):
-                from metadrive.obs.observation_base import ImageObservation
-                image_source = "rgb_camera"
-                assert len(self.vehicles) == 1, "Multi-agent not supported yet!"
-                self.temporary_img_obs = ImageObservation(self.vehicles[DEFAULT_AGENT].config, image_source, False)
-            else:
-                raise ValueError("Not implemented yet!")
-            self.temporary_img_obs.observe(self.vehicles[DEFAULT_AGENT].image_sensors[image_source])
-            return self.temporary_img_obs.get_image()
+        if mode == "rgb_array":
+            assert self.config["use_render"], "You should create a Panda3d window before rendering images!"
+            # if not hasattr(self, "temporary_img_obs"):
+            #     from metadrive.obs.image_obs import ImageObservation
+            #     image_source = "rgb_camera"
+            #     assert len(self.vehicles) == 1, "Multi-agent not supported yet!"
+            #     self.temporary_img_obs = ImageObservation(self.vehicles[DEFAULT_AGENT].config, image_source, False)
+            # # else:
+            # #     raise ValueError("Not implemented yet!")
+            # self.temporary_img_obs.observe(self.vehicles[DEFAULT_AGENT])
+            # return self.temporary_img_obs.get_image()
+            return self.engine.get_window_image(return_bytes=return_bytes)
 
-        # logging.warning("You do not set 'offscreen_render' or 'offscreen_render' to True, so no image will be returned!")
+        # logging.warning("You do not set 'image_observation' or 'image_observation' to True, so no image will be returned!")
         return None
 
     def reset(self, force_seed: Union[None, int] = None):
@@ -348,7 +386,7 @@ class BaseEnv(gym.Env):
         self.episode_lengths = defaultdict(int)
 
         assert (len(self.vehicles) == self.num_agents) or (self.num_agents == -1)
-
+        assert self.config is self.engine.global_config is get_global_config(), "Inconsistent config may bring errors!"
         return self._get_reset_return()
 
     def _get_reset_return(self):
@@ -408,10 +446,13 @@ class BaseEnv(gym.Env):
         time.sleep(2)  # Sleep two seconds
         raise KeyboardInterrupt("'Esc' is pressed. MetaDrive exits now.")
 
-    def capture(self):
-        img = PNMImage()
-        self.engine.win.getScreenshot(img)
-        img.write("main_{}.png".format(time.time()))
+    def capture(self, file_name=None):
+        if not hasattr(self, "_capture_img"):
+            self._capture_img = PNMImage()
+        self.engine.win.getScreenshot(self._capture_img)
+        if file_name is None:
+            file_name = "main_{}.png".format(time.time())
+        self._capture_img.write(file_name)
 
     def for_each_vehicle(self, func, *args, **kwargs):
         return self.agent_manager.for_each_active_agents(func, *args, **kwargs)
@@ -427,7 +468,7 @@ class BaseEnv(gym.Env):
         return ego_v
 
     def get_single_observation(self, vehicle_config: "Config"):
-        if self.config["offscreen_render"]:
+        if self.config["image_observation"]:
             o = ImageStateObservation(vehicle_config)
         else:
             o = LidarStateObservation(vehicle_config)
@@ -534,3 +575,7 @@ class BaseEnv(gym.Env):
     @property
     def episode_step(self):
         return self.engine.episode_step if self.engine is not None else 0
+
+    @property
+    def total_step(self):
+        return self.engine.episode_step if self.engine is not None else 0
