@@ -30,7 +30,7 @@ if __name__ == "__main__":
             if d and info["arrive_dest"]:
                 env.reset()
                 env.current_track_vehicle.expert_takeover = True
-    except:
-        pass
+    except Exception as e:
+        raise e
     finally:
         env.close()
