@@ -1,12 +1,11 @@
 import copy
 from typing import Dict
 
-from gym.spaces import Box, Dict, MultiDiscrete
+from gym.spaces import Box, Dict, MultiDiscrete, Discrete
 
 from metadrive.constants import DEFAULT_AGENT
 from metadrive.manager.base_manager import BaseManager
 from metadrive.policy.AI_protect_policy import AIProtectPolicy
-from metadrive.policy.env_input_policy import EnvInputPolicy
 from metadrive.policy.manual_control_policy import ManualControlPolicy
 
 
@@ -67,30 +66,40 @@ class AgentManager(BaseManager):
                 vehicle_type[v_config["vehicle_model"] if v_config.get("vehicle_model", False) else "default"]
             obj = self.spawn_object(v_type, vehicle_config=v_config)
             ret[agent_id] = obj
-            policy_cls = self.get_policy()
+            policy_cls = self.agent_policy
             self.add_policy(obj.id, policy_cls, obj, self.generate_seed())
         return ret
 
-    def get_policy(self):
+    @property
+    def agent_policy(self):
         # note: agent.id = object id
-        if self.engine.global_config["agent_policy"] is not None:
-            return self.engine.global_config["agent_policy"]
+
         if self.engine.global_config["manual_control"]:
             if self.engine.global_config.get("use_AI_protector", False):
                 policy = AIProtectPolicy
             else:
                 policy = ManualControlPolicy
         else:
-            policy = EnvInputPolicy
+            policy = self.engine.global_config["agent_policy"]
         return policy
 
     def before_reset(self):
         if not self.INITIALIZED:
             super(AgentManager, self).__init__()
             self.INITIALIZED = True
-        super(AgentManager, self).before_reset()
+
         self.episode_created_agents = None
 
+        if not self.engine.replay_episode:
+            for v in self.dying_agents.values():
+                self._remove_vehicle(v)
+
+        for v in self.get_vehicle_list():
+            if hasattr(v, "before_reset"):
+                v.before_reset()
+
+        super(AgentManager, self).before_reset()
+
     def reset(self):
         """
         Agent manager is really initialized after the BaseVehicle Instances are created
@@ -128,13 +137,14 @@ class AgentManager(BaseManager):
             self.observations[vehicle.name] = self._init_observations[agent_id]
             obs_space = self._init_observation_spaces[agent_id]
             self.observation_spaces[vehicle.name] = obs_space
-            if not self.engine.global_config["offscreen_render"]:
+            if not self.engine.global_config["image_observation"]:
                 assert isinstance(obs_space, Box)
             else:
                 assert isinstance(obs_space, Dict), "Multi-agent observation should be gym.Dict"
             action_space = self._init_action_spaces[agent_id]
             self.action_spaces[vehicle.name] = action_space
-            assert isinstance(action_space, Box) or isinstance(action_space, MultiDiscrete)
+            assert isinstance(action_space, Box) or isinstance(action_space,
+                                                               MultiDiscrete) or isinstance(action_space, Discrete)
         self.next_agent_count = len(init_vehicles)
 
     def set_state(self, state: dict, old_name_to_current=None):
@@ -214,7 +224,8 @@ class AgentManager(BaseManager):
         self._agents_finished_this_frame = dict()
         step_infos = {}
         for agent_id in self.active_agents.keys():
-            policy = self.engine.get_policy(self._agent_to_object[agent_id])
+            policy = self.get_policy(self._agent_to_object[agent_id])
+            assert policy is not None, "No policy is set for agent {}".format(agent_id)
             action = policy.act(agent_id)
             step_infos[agent_id] = policy.get_action_info()
             step_infos[agent_id].update(self.get_agent(agent_id).before_step(action))
@@ -280,7 +291,7 @@ class AgentManager(BaseManager):
         """
         Return Map<agent_id, BaseVehicle>
         """
-        if hasattr(self, "engine") and self.engine.replay_episode:
+        if hasattr(self, "engine") and self.engine is not None and self.engine.replay_episode:
             return self.engine.replay_manager.replay_agents
         else:
             return {self._object_to_agent[k]: v for k, v in self._active_objects.items()}
