@@ -11,8 +11,8 @@ class BaseManager(Randomizable):
     def __init__(self):
         from metadrive.engine.engine_utils import get_engine, engine_initialized
         assert engine_initialized(), "You should not create manager before the initialization of BaseEngine"
-        self.engine = get_engine()
-        Randomizable.__init__(self, self.engine.global_random_seed)
+        # self.engine = get_engine()
+        Randomizable.__init__(self, get_engine().global_random_seed)
         self.spawned_objects = {}
 
     @property
@@ -58,7 +58,7 @@ class BaseManager(Randomizable):
         """
         Destroy manager
         """
-        self.engine = None
+        # self.engine = None
         self.spawned_objects = None
 
     def spawn_object(self, object_class, **kwargs):
@@ -93,6 +93,9 @@ class BaseManager(Randomizable):
     def add_policy(self, object_id, policy_class, *policy_args, **policy_kwargs):
         return self.engine.add_policy(object_id, policy_class, *policy_args, **policy_kwargs)
 
+    def get_policy(self, object_id):
+        return self.engine.get_policy(object_id)
+
     def get_state(self):
         """This function will be called by RecordManager to collect manager state, usually some mappings"""
         assert self.episode_step == 0, "This func can only be called after env.reset() without any env.step() called"
@@ -118,3 +121,8 @@ class BaseManager(Randomizable):
     @property
     def class_name(self):
         return self.__class__.__name__
+
+    @property
+    def engine(self):
+        from metadrive.engine.engine_utils import get_engine
+        return get_engine()
