@@ -1,8 +1,8 @@
-import logging
+import pickle
+
 from tqdm import tqdm
 
 from metadrive.component.map.pg_map import PGMap, MapGenerateMethod
-import pickle
 from metadrive.manager.base_manager import BaseManager
 from metadrive.utils.utils import get_time_str
 
@@ -23,7 +23,8 @@ class PGMapManager(BaseManager):
         self.maps = {_seed: None for _seed in range(start_seed, start_seed + env_num)}
 
     def spawn_object(self, object_class, *args, **kwargs):
-        map = self.engine.spawn_object(object_class, auto_fill_random_seed=False, *args, **kwargs)
+        # Note: Map instance should not be reused / recycled.
+        map = self.engine.spawn_object(object_class, auto_fill_random_seed=False, force_spawn=True, *args, **kwargs)
         return map
 
     def load_map(self, map):
@@ -99,7 +100,7 @@ class PGMapManager(BaseManager):
     def load_all_maps(self, file_name):
         if self.current_map is not None:
             self.unload_map(self.current_map)
-        with open(file_name, "rb+") as file:
+        with open(file_name, "rb") as file:
             loaded_map_data = pickle.load(file)
         map_seeds = list(loaded_map_data.keys())
         start_seed = min(map_seeds)
@@ -123,6 +124,12 @@ class PGMapManager(BaseManager):
         self.reset()
         return loaded_map_data
 
+    def clear_objects(self, *args, **kwargs):
+        """
+        As Map instance should not be recycled, we will forcefully destroy useless map instances.
+        """
+        return super(PGMapManager, self).clear_objects(force_destroy=True, *args, **kwargs)
+
 
 # For compatibility
 MapManager = PGMapManager
