@@ -84,22 +84,25 @@ class StateObservation(ObservationBase):
             lateral_to_right /= total_width
             info += [clip(lateral_to_left, 0.0, 1.0), clip(lateral_to_right, 0.0, 1.0)]
 
-        current_reference_lane = vehicle.navigation.current_ref_lanes[-1]
-        info += [
+        if vehicle.navigation is None or vehicle.navigation.current_ref_lanes is None:
+            info += [0] * 5
+        else:
+            current_reference_lane = vehicle.navigation.current_ref_lanes[-1]
+            info += [
 
-            # The angular difference between vehicle's heading and the lane heading at this location.
-            vehicle.heading_diff(current_reference_lane),
+                # The angular difference between vehicle's heading and the lane heading at this location.
+                vehicle.heading_diff(current_reference_lane),
 
-            # The velocity of target vehicle
-            clip((vehicle.speed + 1) / (vehicle.max_speed + 1), 0.0, 1.0),
+                # The velocity of target vehicle
+                clip((vehicle.speed_km_h + 1) / (vehicle.max_speed_km_h + 1), 0.0, 1.0),
 
-            # Current steering
-            clip((vehicle.steering / vehicle.MAX_STEERING + 1) / 2, 0.0, 1.0),
+                # Current steering
+                clip((vehicle.steering / vehicle.MAX_STEERING + 1) / 2, 0.0, 1.0),
 
-            # The normalized actions at last steps
-            clip((vehicle.last_current_action[0][0] + 1) / 2, 0.0, 1.0),
-            clip((vehicle.last_current_action[0][1] + 1) / 2, 0.0, 1.0)
-        ]
+                # The normalized actions at last steps
+                clip((vehicle.last_current_action[0][0] + 1) / 2, 0.0, 1.0),
+                clip((vehicle.last_current_action[0][1] + 1) / 2, 0.0, 1.0)
+            ]
 
         # Current angular acceleration (yaw rate)
         heading_dir_last = vehicle.last_heading_dir
