@@ -1,7 +1,7 @@
-import math
 from collections import deque
 
 import gym
+import math
 import numpy as np
 
 from metadrive.component.vehicle.base_vehicle import BaseVehicle
@@ -67,7 +67,7 @@ class TopDownMultiChannel(TopDownObservation):
         self.canvas_past_pos = pygame.Surface(self.resolution)  # A local view
 
     def reset(self, env, vehicle=None):
-        self.engine = env.engine
+        # self.engine = env.engine
         self.road_network = env.current_map.road_network
         self.target_vehicle = vehicle
         self._should_draw_map = True
