@@ -1,23 +1,18 @@
 import copy
-from metadrive.utils.interpolating_line import InterpolatingLine
-
 from collections import deque, namedtuple
-import copy
 from typing import Optional, Union, Iterable
 
-import cv2
 import numpy as np
 
 from metadrive.component.map.waymo_map import WaymoMap
 from metadrive.constants import Decoration, TARGET_VEHICLES
-from metadrive.constants import LineColor
 from metadrive.constants import WaymoLaneProperty
-
 from metadrive.engine.engine_utils import get_engine
 from metadrive.obs.top_down_obs_impl import WorldSurface, VehicleGraphics, LaneGraphics
+from metadrive.utils.interpolating_line import InterpolatingLine
 from metadrive.utils.map_utils import is_map_related_instance
 from metadrive.utils.utils import import_pygame
-from metadrive.utils.waymo_utils.waymo_utils import RoadLineType, RoadEdgeType, convert_polyline_to_metadrive
+from metadrive.utils.waymo_utils.waymo_utils import convert_polyline_to_metadrive
 
 pygame = import_pygame()
 
@@ -34,6 +29,7 @@ def draw_top_down_map(
     reverse_color=False,
     road_color=color_white
 ) -> Optional[Union[np.ndarray, pygame.Surface]]:
+    import cv2
     film_size = film_size or map.film_size
     surface = WorldSurface(film_size, 0, pygame.Surface(film_size))
     if reverse_color:
@@ -143,6 +139,7 @@ class TopDownRenderer:
         show_agent_name=False,
         camera_position=None,
         track_target_vehicle=False,
+        draw_target_vehicle_trajectory=False,
         **kwargs
         # current_track_vehicle=None
     ):
@@ -150,14 +147,18 @@ class TopDownRenderer:
         self.position = camera_position
         self.track_target_vehicle = track_target_vehicle
         self.show_agent_name = show_agent_name
+        self.draw_target_vehicle_trajectory = draw_target_vehicle_trajectory
+
         if self.show_agent_name:
             pygame.init()
-        self.engine = get_engine()
+
+        # self.engine = get_engine()
         # self._screen_size = screen_size
         self.pygame_font = None
         self.map = self.engine.current_map
         self.stack_frames = deque(maxlen=num_stack)
         self.history_objects = deque(maxlen=num_stack)
+        self.history_target_vehicle = []
         self.history_smooth = history_smooth
         # self.current_track_vehicle = current_track_vehicle
         if self.track_target_vehicle:
@@ -227,6 +228,19 @@ class TopDownRenderer:
         this_frame_objects = self._append_frame_objects(objects)
         self.history_objects.append(this_frame_objects)
 
+        if self.draw_target_vehicle_trajectory:
+            self.history_target_vehicle.append(
+                history_object(
+                    name=self.current_track_vehicle.name,
+                    heading_theta=self.current_track_vehicle.heading_theta,
+                    WIDTH=self.current_track_vehicle.top_down_width,
+                    LENGTH=self.current_track_vehicle.top_down_length,
+                    position=self.current_track_vehicle.position,
+                    color=self.current_track_vehicle.top_down_color,
+                    done=False
+                )
+            )
+
         self._handle_event()
         self.refresh()
         self._draw(*args, **kwargs)
@@ -278,6 +292,9 @@ class TopDownRenderer:
         # self._runtime_output = self._background_canvas.copy()
         self._background_size = tuple(self._background_canvas.get_size())
 
+        self.history_objects.clear()
+        self.history_target_vehicle.clear()
+
     @property
     def current_track_vehicle(self):
         return self.engine.current_track_vehicle
@@ -304,6 +321,7 @@ class TopDownRenderer:
         """
         if len(self.history_objects) == 0:
             return
+
         for i, objects in enumerate(self.history_objects):
             i = len(self.history_objects) - i
             if self.history_smooth != 0 and (i % self.history_smooth != 0):
@@ -322,6 +340,27 @@ class TopDownRenderer:
                     draw_countour=False
                 )
 
+        # Draw the whole trajectory of ego vehicle with no gradient colors:
+        if self.draw_target_vehicle_trajectory:
+            for i, v in enumerate(self.history_target_vehicle):
+                i = len(self.history_target_vehicle) - i
+                if self.history_smooth != 0 and (i % self.history_smooth != 0):
+                    continue
+                c = v.color
+                h = v.heading_theta
+                h = h if abs(h) > 2 * np.pi / 180 else 0
+                x = abs(int(i))
+                alpha_f = min(x / len(self.history_target_vehicle), 0.5)
+                # alpha_f = 0
+                VehicleGraphics.display(
+                    vehicle=v,
+                    surface=self._runtime_canvas,
+                    heading=h,
+                    color=(c[0] + alpha_f * (255 - c[0]), c[1] + alpha_f * (255 - c[1]), c[2] + alpha_f * (255 - c[2])),
+                    draw_countour=False
+                )
+
+        # Draw current vehicle with black contour
         # Use this line if you wish to draw "future" trajectory.
         # i is the index of vehicle that we will render a black box for it.
         # i = int(len(self.history_vehicles) / 2)
@@ -416,3 +455,8 @@ class TopDownRenderer:
                 if event.key == pygame.K_ESCAPE:
                     import sys
                     sys.exit()
+
+    @property
+    def engine(self):
+        from metadrive.engine.engine_utils import get_engine
+        return get_engine()
