@@ -1,6 +1,5 @@
-import logging
-
 import numpy as np
+
 from metadrive.component.vehicle_module.PID_controller import PIDController
 from metadrive.policy.base_policy import BasePolicy
 from metadrive.policy.manual_control_policy import ManualControlPolicy
@@ -200,10 +199,10 @@ class IDMPolicy(BasePolicy):
     LANE_CHANGE_SPEED_INCREASE = 10
     SAFE_LANE_CHANGE_DISTANCE = 15
     MAX_LONG_DIST = 30
-    MAX_SPEED = 100
+    MAX_SPEED = 100  # km/h
 
     # Normal speed
-    NORMAL_SPEED = 30
+    NORMAL_SPEED = 30  # km/h
 
     # Creep Speed
     CREEP_SPEED = 5
@@ -292,7 +291,7 @@ class IDMPolicy(BasePolicy):
     def acceleration(self, front_obj, dist_to_front) -> float:
         ego_vehicle = self.control_object
         ego_target_speed = not_zero(self.target_speed, 0)
-        acceleration = self.ACC_FACTOR * (1 - np.power(max(ego_vehicle.speed, 0) / ego_target_speed, self.DELTA))
+        acceleration = self.ACC_FACTOR * (1 - np.power(max(ego_vehicle.speed_km_h, 0) / ego_target_speed, self.DELTA))
         if front_obj and (not self.disable_idm_deceleration):
             d = dist_to_front
             speed_diff = self.desired_gap(ego_vehicle, front_obj) / not_zero(d)
@@ -303,9 +302,9 @@ class IDMPolicy(BasePolicy):
         d0 = self.DISTANCE_WANTED
         tau = self.TIME_WANTED
         ab = -self.ACC_FACTOR * self.DEACC_FACTOR
-        dv = np.dot(ego_vehicle.velocity - front_obj.velocity, ego_vehicle.heading) if projected \
-            else ego_vehicle.speed - front_obj.speed
-        d_star = d0 + ego_vehicle.speed * tau + ego_vehicle.speed * dv / (2 * np.sqrt(ab))
+        dv = np.dot(ego_vehicle.velocity_km_h - front_obj.velocity_km_h, ego_vehicle.heading) if projected \
+            else ego_vehicle.speed_km_h - front_obj.speed_km_h
+        d_star = d0 + ego_vehicle.speed_km_h * tau + ego_vehicle.speed_km_h * dv / (2 * np.sqrt(ab))
         return d_star
 
     def reset(self):
@@ -363,15 +362,15 @@ class IDMPolicy(BasePolicy):
                                current_lanes[self.routing_target_lane.index[-1] + 1]
 
         # lane follow or active change lane/overtake for high driving speed
-        if abs(self.control_object.speed - self.NORMAL_SPEED) > 3 and surrounding_objects.has_front_object(
-        ) and abs(surrounding_objects.front_object().speed -
+        if abs(self.control_object.speed_km_h - self.NORMAL_SPEED) > 3 and surrounding_objects.has_front_object(
+        ) and abs(surrounding_objects.front_object().speed_km_h -
                   self.NORMAL_SPEED) > 3 and self.overtake_timer > self.LANE_CHANGE_FREQ:
             # may lane change
-            right_front_speed = surrounding_objects.right_front_object().speed if surrounding_objects.has_right_front_object() else self.MAX_SPEED \
+            right_front_speed = surrounding_objects.right_front_object().speed_km_h if surrounding_objects.has_right_front_object() else self.MAX_SPEED \
                 if surrounding_objects.right_lane_exist() and surrounding_objects.right_front_min_distance() > self.SAFE_LANE_CHANGE_DISTANCE and surrounding_objects.right_back_min_distance() > self.SAFE_LANE_CHANGE_DISTANCE else None
-            front_speed = surrounding_objects.front_object().speed if surrounding_objects.has_front_object(
+            front_speed = surrounding_objects.front_object().speed_km_h if surrounding_objects.has_front_object(
             ) else self.MAX_SPEED
-            left_front_speed = surrounding_objects.left_front_object().speed if surrounding_objects.has_left_front_object() else self.MAX_SPEED \
+            left_front_speed = surrounding_objects.left_front_object().speed_km_h if surrounding_objects.has_left_front_object() else self.MAX_SPEED \
                 if surrounding_objects.left_lane_exist() and surrounding_objects.left_front_min_distance() > self.SAFE_LANE_CHANGE_DISTANCE and surrounding_objects.left_back_min_distance() > self.SAFE_LANE_CHANGE_DISTANCE else None
             if left_front_speed is not None and left_front_speed - front_speed > self.LANE_CHANGE_SPEED_INCREASE:
                 # left overtake has a high priority
@@ -392,6 +391,7 @@ class IDMPolicy(BasePolicy):
 
 
 class ManualControllableIDMPolicy(IDMPolicy):
+    """If human is not taking over, then use IDM policy."""
     def __init__(self, *args, **kwargs):
         super(ManualControllableIDMPolicy, self).__init__(*args, **kwargs)
         self.manual_control_policy = ManualControlPolicy(*args, **kwargs)
@@ -404,19 +404,15 @@ class ManualControllableIDMPolicy(IDMPolicy):
             return super(ManualControllableIDMPolicy, self).act(agent_id)
 
 
-class WaymoIDMPolicy(IDMPolicy):
+class TrajectoryIDMPOlicy(IDMPolicy):
+    """This policy is customized for the traffic car in Waymo environment. (Ego car is not included!)"""
     NORMAL_SPEED = 40
     WAYMO_IDM_MAX_DIST = 20
 
-    # DEACC_FACTOR = -20
-    # ACC_FACTOR = 0.5
-    # TIME_WANTED = 80
-    # DISTANCE_WANTED = 55.0
-
-    def __init__(self, control_object, random_seed, traj_to_follow, policy_index):
-        super(WaymoIDMPolicy, self).__init__(control_object=control_object, random_seed=random_seed)
+    def __init__(self, control_object, random_seed, traj_to_follow=None, policy_index=None):
+        super(TrajectoryIDMPOlicy, self).__init__(control_object=control_object, random_seed=random_seed)
         self.policy_index = policy_index
-        self.traj_to_follow = traj_to_follow
+        self.traj_to_follow = self.engine.map_manager.current_sdc_route if traj_to_follow is None else traj_to_follow
         self.target_speed = self.NORMAL_SPEED
         self.routing_target_lane = self.traj_to_follow
         self.available_routing_index_range = None
@@ -470,14 +466,15 @@ class WaymoIDMPolicy(IDMPolicy):
         return [steering, acc]
 
 
-class EgoWaymoIDMPolicy(IDMPolicy):
+class _EgoWaymoIDMPolicy(IDMPolicy):
     """
-    This policy is for Vehicles using navigation module, especially, ego car
+    DeprecationWarning("This policy is merged with WaymoIDM Policy now")
     """
     NORMAL_SPEED = 30
 
     def __init__(self, control_object, random_seed):
-        super(EgoWaymoIDMPolicy, self).__init__(control_object=control_object, random_seed=random_seed)
+        raise DeprecationWarning("This policy is merged with WaymoIDM Policy now")
+        super(_EgoWaymoIDMPolicy, self).__init__(control_object=control_object, random_seed=random_seed)
         self.target_speed = self.NORMAL_SPEED
         self.routing_target_lane = None
         self.available_routing_index_range = None
@@ -488,6 +485,7 @@ class EgoWaymoIDMPolicy(IDMPolicy):
         self.lateral_pid = PIDController(0.3, .0, 0.0)
 
     def steering_control(self, target_lane) -> float:
+        """Identical to the steering control of WaymoIDMPolicy"""
         # heading control following a lateral distance control
         ego_vehicle = self.control_object
         long, lat = target_lane.local_coordinates(ego_vehicle.position)
@@ -513,3 +511,9 @@ class EgoWaymoIDMPolicy(IDMPolicy):
             )
             return True
         return False
+
+
+# Currently, all policies are the same
+EgoWaymoIDMPolicy = TrajectoryIDMPOlicy
+WaymoIDMPolicy = TrajectoryIDMPOlicy
+NuPlanIDMPolicy = TrajectoryIDMPOlicy
