@@ -206,9 +206,9 @@ def test_ma_intersection_reset():
                         v.navigation.get_current_lane_width() / 2 >= lat >=
                         (0.5 - v.navigation.get_current_lane_num()) * v.navigation.get_current_lane_width()
                     )
-                    if not v.arrive_destination:
+                    if not env._is_arrive_destination(v):
                         print('sss')
-                    assert v.arrive_destination
+                    assert env._is_arrive_destination(v)
 
                 act = {k: [0, 0] for k in env.vehicles.keys()}
                 o, r, d, i = _act(env, act)
@@ -471,6 +471,7 @@ def test_ma_intersection_reward_sign():
             return safe_places
 
     env = TestEnv({"num_agents": 1})
+    np.random.seed(10)
     try:
         _check_spaces_before_reset(env)
         obs = env.reset()
@@ -699,7 +700,7 @@ if __name__ == '__main__':
     test_ma_intersection_env()
     # test_ma_intersection_horizon()
     # test_ma_intersection_reset()
-    test_ma_intersection_reward_done_alignment()
+    # test_ma_intersection_reward_done_alignment()
     # test_ma_intersection_close_spawn()
     # test_ma_intersection_reward_sign()
     # test_ma_intersection_init_space()
