@@ -205,9 +205,9 @@ def test_ma_toll_reset():
                         v.navigation.get_current_lane_width() / 2 >= lat >=
                         (0.5 - v.navigation.get_current_lane_num()) * v.navigation.get_current_lane_width()
                     )
-                    if not v.arrive_destination:
+                    if not env._is_arrive_destination(v):
                         print('sss')
-                    assert v.arrive_destination
+                    assert env._is_arrive_destination(v)
 
                 act = {k: [0, 0] for k in env.vehicles.keys()}
                 o, r, d, i = _act(env, act)
@@ -566,7 +566,7 @@ def test_ma_toll_no_short_episode():
         env.close()
 
 
-def test_ma_toll_horizon_termination():
+def test_ma_toll_horizon_termination(vis=False):
     # test horizon
     env = MultiAgentTollgateEnv({
         "horizon": 100,
@@ -590,6 +590,10 @@ def test_ma_toll_horizon_termination():
                         if v_id in env.vehicles:
                             env.vehicles[v_id].set_static(True)
                 obs, r, d, i = _act(env, act)
+
+                if vis:
+                    env.render("topdown")
+
                 if step == 0 or step == 1:
                     assert not any(d.values())
 
@@ -704,12 +708,12 @@ if __name__ == '__main__':
     # test_ma_toll_env()
     # test_ma_toll_horizon()
     # test_ma_toll_reset()
-    test_ma_toll_reward_done_alignment_1()
+    # test_ma_toll_reward_done_alignment_1()
     # test_ma_toll_close_spawn()
     # test_ma_toll_reward_sign()
     # test_ma_toll_init_space()
     # test_ma_toll_no_short_episode()
-    # test_ma_toll_horizon_termination()
+    test_ma_toll_horizon_termination(vis=True)
     # test_ma_toll_40_agent_reset_after_respawn()
     # test_ma_no_reset_error()
     # test_randomize_spawn_place()
