@@ -5,15 +5,7 @@ from metadrive.utils import setup_logger
 def test_traffic_respawn(vis=False):
     setup_logger(vis)
 
-    env = MetaDriveEnv(
-        {
-            "environment_num": 1,
-            "manual_control": vis,
-            "use_render": vis,
-            "use_topdown": True,
-            "traffic_mode": "respawn"
-        }
-    )
+    env = MetaDriveEnv({"environment_num": 1, "manual_control": vis, "use_render": vis, "traffic_mode": "respawn"})
     env.reset()
     try:
         for i in range(1, 3000):
