@@ -12,13 +12,14 @@ from metadrive.engine.core.physics_world import PhysicsWorld
 
 
 class TestBlock(ShowBase.ShowBase):
-    def __init__(self, debug=False):
+    def __init__(self, debug=False, window_type="onscreen"):
         self.debug = debug
-        super(TestBlock, self).__init__(windowType="onscreen")
+        super(TestBlock, self).__init__(windowType=window_type)
         self.setBackgroundColor(BKG_COLOR)
-        self.setFrameRateMeter(True)
-        self.cam.setPos(0, 0, 300)
-        self.cam.lookAt(0, 0, 0)
+        if window_type != "none":
+            self.setFrameRateMeter(True)
+            self.cam.setPos(0, 0, 300)
+            self.cam.lookAt(0, 0, 0)
         self.map = None
         self.worldNP = None
         self.world = None
@@ -105,7 +106,9 @@ class TestBlock(ShowBase.ShowBase):
         line_seg.moveTo(start_p)
         line_seg.drawTo(end_p)
         line_seg.setThickness(thickness)
-        NodePath(line_seg.create(False)).reparentTo(self.render)
+        np = NodePath(line_seg.create(False))
+        np.reparentTo(self.render)
+        # TODO(PZH): NodePath is not registered.
 
     def show_bounding_box(self, road_network):
         bound_box = road_network.get_bounding_box()
