@@ -1,6 +1,6 @@
-from metadrive.component.blocks.curve import Curve
-from metadrive.component.blocks.first_block import FirstPGBlock
-from metadrive.component.road.road_network import RoadNetwork
+from metadrive.component.pgblock.curve import Curve
+from metadrive.component.pgblock.first_block import FirstPGBlock
+from metadrive.component.road_network.node_road_network import NodeRoadNetwork
 from metadrive.engine.asset_loader import initialize_asset_loader
 from metadrive.tests.vis_block.vis_block_base import TestBlock
 
@@ -9,7 +9,7 @@ if __name__ == "__main__":
 
     initialize_asset_loader(test)
 
-    global_network = RoadNetwork()
+    global_network = NodeRoadNetwork()
     curve = FirstPGBlock(global_network, 3.0, 1, test.render, test.world, 1)
     for i in range(1, 13):
         curve = Curve(i, curve.get_socket(0), global_network, i)
