@@ -1,7 +1,7 @@
-from metadrive.component.blocks.curve import Curve
-from metadrive.component.blocks.first_block import FirstPGBlock
-from metadrive.component.blocks.intersection import InterSection
-from metadrive.component.road.road_network import RoadNetwork
+from metadrive.component.pgblock.curve import Curve
+from metadrive.component.pgblock.first_block import FirstPGBlock
+from metadrive.component.pgblock.intersection import InterSection
+from metadrive.component.road_network.node_road_network import NodeRoadNetwork
 from metadrive.tests.vis_block.vis_block_base import TestBlock
 
 if __name__ == "__main__":
@@ -10,8 +10,8 @@ if __name__ == "__main__":
 
     initialize_asset_loader(test)
 
-    global_network = RoadNetwork()
-    first = FirstPGBlock(global_network, 3.0, 2, test.render, test.world, 1)
+    global_network = NodeRoadNetwork()
+    first = FirstPGBlock(global_network, 3.0, 2, test.render, test.world, 20)
 
     intersection = InterSection(3, first.get_socket(0), global_network, 1)
     print(intersection.construct_block(test.render, test.world))
