@@ -1,8 +1,8 @@
-from metadrive.component.blocks.curve import Curve
-from metadrive.component.blocks.first_block import FirstPGBlock
-from metadrive.component.blocks.straight import Straight
-from metadrive.component.blocks.t_intersection import TInterSection
-from metadrive.component.road.road_network import RoadNetwork
+from metadrive.component.pgblock.curve import Curve
+from metadrive.component.pgblock.first_block import FirstPGBlock
+from metadrive.component.pgblock.straight import Straight
+from metadrive.component.pgblock.t_intersection import TInterSection
+from metadrive.component.road_network.node_road_network import NodeRoadNetwork
 from metadrive.tests.vis_block.vis_block_base import TestBlock
 
 if __name__ == "__main__":
@@ -11,7 +11,7 @@ if __name__ == "__main__":
 
     initialize_asset_loader(test)
 
-    global_network = RoadNetwork()
+    global_network = NodeRoadNetwork()
     first = FirstPGBlock(global_network, 3.0, 2, test.render, test.world, 1)
 
     curve = Curve(1, first.get_socket(0), global_network, 1)
