@@ -14,7 +14,7 @@ if __name__ == "__main__":
             },
             "manual_control": True,
             "use_render": False,
-            "offscreen_render": True,  # it is a switch telling metadrive to use rgb as observation
+            "image_observation": True,  # it is a switch telling metadrive to use rgb as observation
             "rgb_clip": True,  # clip rgb to range(0,1) instead of (0, 255)
             # "pstats": True,
         }
@@ -22,7 +22,7 @@ if __name__ == "__main__":
     env.reset()
     # print m to capture rgb observation
     env.engine.accept(
-        "m", env.vehicle.image_sensors[env.vehicle.config["image_source"]].save_image, extraArgs=[env.vehicle]
+        "m", env.vehicle.get_camera(env.vehicle.config["image_source"]).save_image, extraArgs=[env.vehicle]
     )
     start = time.time()
     for i in range(1, 100000):
