@@ -10,10 +10,9 @@ if __name__ == "__main__":
             "start_seed": 4,
             "manual_control": True,
             "use_render": True,
-            "offscreen_render": True,
+            "image_observation": True,
             "rgb_clip": True,
             "vehicle_config": dict(depth_camera=(200, 88, False), image_source="depth_camera"),
-            "headless_machine_render": False,
             "map_config": {
                 BaseMap.GENERATE_TYPE: MapGenerateMethod.BIG_BLOCK_NUM,
                 BaseMap.GENERATE_CONFIG: 12,
@@ -23,7 +22,7 @@ if __name__ == "__main__":
         }
     )
     env.reset()
-    env.engine.accept("m", env.vehicle.image_sensors["depth_camera"].save_image, extraArgs=[env.vehicle, "debug.jpg"])
+    env.engine.accept("m", env.vehicle.get_camera("depth_camera").save_image, extraArgs=[env.vehicle, "debug.jpg"])
 
     for i in range(1, 100000):
         o, r, d, info = env.step([0, 1])
@@ -33,7 +32,7 @@ if __name__ == "__main__":
             #     ObservationType.show_gray_scale_array(o["image"][:, :, i])
             env.render(text={"can you see me": i})
         if d:
-            env.vehicle.image_sensors["rgb_camera"].save_image(env.vehicle)
+            env.vehicle.get_camera("rgb_camera").save_image(env.vehicle)
             print("Reset")
             env.reset()
     env.close()
