@@ -1,7 +1,7 @@
 import pickle
 
 from metadrive.envs.metadrive_env import MetaDriveEnv
-from metadrive.utils import recursive_equal, setup_logger
+from metadrive.utils import setup_logger
 
 
 def vis_gen_map_read():
@@ -23,7 +23,7 @@ def vis_gen_map_read():
         env.close()
 
         # Check load
-        with open("test_10maps.pickle", "rb+") as f:
+        with open("test_10maps.pickle", "rb") as f:
             restored_data = pickle.load(f)
 
         env = MetaDriveEnv(restore_config)
