@@ -9,13 +9,12 @@ if __name__ == "__main__":
             "image_source": "mini_map",
             "manual_control": True,
             "use_render": True,
-            "offscreen_render": True,
+            "image_observation": True,
             "rgb_clip": True,
-            "headless_machine_render": False
         }
     )
     env.reset()
-    env.engine.accept("m", env.vehicle.image_sensors[env.config["image_source"]].save_image)
+    env.engine.accept("m", env.vehicle.get_camera([env.config["image_source"]]).save_image)
 
     for i in range(1, 100000):
         o, r, d, info = env.step([0, 1])
