@@ -14,7 +14,7 @@ if __name__ == "__main__":
             "image_source": "mini_map",
             "manual_control": True,
             "use_render": True,
-            "offscreen_render": False,
+            "image_observation": False,
             "decision_repeat": 5,
             "rgb_clip": True,
             "map_config": {
@@ -29,5 +29,5 @@ if __name__ == "__main__":
     env.reset()
     for i in range(1, 100000):
         o, r, d, info = env.step([0, 1])
-        env.render(text={"Frame": i, "Speed": env.vehicle.speed})
+        env.render(text={"Frame": i, "Speed": env.vehicle.speed_km_h})
     env.close()
