@@ -1,12 +1,12 @@
 from metadrive.envs.marl_envs.marl_parking_lot import MultiAgentParkingLotEnv
-import cv2
 
 
 def vis_ma_parking_lot_env():
+    import cv2
     env = MultiAgentParkingLotEnv(
         {
             "use_render": False,
-            "offscreen_render": True,
+            "image_observation": True,
             # it is a switch telling metadrive to use rgb as observation
             "rgb_clip": True,  # clip rgb to range(0,1) instead of (0, 255)
             "delay_done": 0,
