@@ -1,7 +1,7 @@
-import math
 import time
 from typing import Tuple
 
+import math
 import numpy as np
 
 number_pos_inf = float("inf")
@@ -9,7 +9,7 @@ number_neg_inf = float("-inf")
 
 
 def safe_clip(array, min_val, max_val):
-    array = np.nan_to_num(array.astype(np.float), copy=False, nan=0.0, posinf=max_val, neginf=min_val)
+    array = np.nan_to_num(array.astype(np.float64), copy=False, nan=0.0, posinf=max_val, neginf=min_val)
     return np.clip(array, min_val, max_val).astype(np.float64)
 
 
@@ -27,7 +27,10 @@ def safe_clip_for_small_array(array, min_val, max_val):
 
 
 def wrap_to_pi(x: float) -> float:
-    return ((x + np.pi) % (2 * np.pi)) - np.pi
+    angles = x
+    angles %= 2 * np.pi
+    angles -= 2 * np.pi * (angles > np.pi)
+    return angles
 
 
 def get_vertical_vector(vector: np.array):
@@ -66,9 +69,10 @@ def distance_greater(vec1, vec2, length):
 
 
 def get_laser_end(lidar_range, perceive_distance, laser_index, heading_theta, vehicle_position_x, vehicle_position_y):
+    angle = lidar_range[laser_index] + heading_theta
     return (
-        perceive_distance * math.cos(lidar_range[laser_index] + heading_theta) + vehicle_position_x,
-        perceive_distance * math.sin(lidar_range[laser_index] + heading_theta) + vehicle_position_y
+        perceive_distance * math.cos(angle) + vehicle_position_x,
+        perceive_distance * math.sin(angle) + vehicle_position_y
     )
 
 
