@@ -61,6 +61,9 @@ class Space:
         # By default, assume identity is JSONable
         return sample_n
 
+    def __del__(self):
+        del self.np_random
+
 
 class Dict(Space):
     """
@@ -107,7 +110,8 @@ class Dict(Space):
         super(Dict, self).__init__(None, None)  # None for shape and dtype, since it'll require special handling
 
     def seed(self, seed=None):
-        [space.seed(seed) for space in self.spaces.values()]
+        for space in self.spaces.values():
+            space.seed(seed)
 
     def sample(self):
         return OrderedDict([(k, space.sample()) for k, space in self.spaces.items()])
@@ -222,7 +226,7 @@ class VehicleParameterSpace:
         max_engine_force=ConstantSpace(800),
         max_brake_force=ConstantSpace(150),
         max_steering=ConstantSpace(40),
-        max_speed=ConstantSpace(80),
+        max_speed_km_h=ConstantSpace(80),
     )
     STATIC_DEFAULT_VEHICLE = STATIC_BASE_VEHICLE
 
@@ -231,7 +235,7 @@ class VehicleParameterSpace:
         max_engine_force=BoxSpace(750, 850),
         max_brake_force=BoxSpace(80, 180),
         max_steering=ConstantSpace(40),
-        max_speed=ConstantSpace(80),
+        max_speed_km_h=ConstantSpace(80),
     )
     DEFAULT_VEHICLE = BASE_VEHICLE
 
@@ -240,28 +244,28 @@ class VehicleParameterSpace:
         max_engine_force=BoxSpace(350, 550),
         max_brake_force=BoxSpace(35, 80),
         max_steering=ConstantSpace(50),
-        max_speed=ConstantSpace(80),
+        max_speed_km_h=ConstantSpace(80),
     )
     M_VEHICLE = dict(
         wheel_friction=ConstantSpace(0.75),
         max_engine_force=BoxSpace(650, 850),
         max_brake_force=BoxSpace(60, 150),
         max_steering=ConstantSpace(45),
-        max_speed=ConstantSpace(80),
+        max_speed_km_h=ConstantSpace(80),
     )
     L_VEHICLE = dict(
         wheel_friction=ConstantSpace(0.8),
         max_engine_force=BoxSpace(450, 650),
         max_brake_force=BoxSpace(60, 120),
         max_steering=ConstantSpace(40),
-        max_speed=ConstantSpace(80),
+        max_speed_km_h=ConstantSpace(80),
     )
     XL_VEHICLE = dict(
         wheel_friction=ConstantSpace(0.7),
         max_engine_force=BoxSpace(500, 700),
         max_brake_force=BoxSpace(50, 100),
         max_steering=ConstantSpace(35),
-        max_speed=ConstantSpace(80),
+        max_speed_km_h=ConstantSpace(80),
     )
 
 
@@ -305,7 +309,7 @@ class BlockParameterSpace:
     }
     BOTTLENECK_PARAMETER = {
         Parameter.length: BoxSpace(min=20, max=50),  # the length of straigh part
-        Parameter.lane_num: DiscreteSpace(min=1, max=2),  # the lane num increased or descreased now 1-2
+        Parameter.lane_num: DiscreteSpace(min=1, max=2),  # the lane num increased or decreased now 1-2
         "bottle_len": ConstantSpace(20),
         "solid_center_line": ConstantSpace(0)  # bool, turn on yellow line or not
     }
